package de.hdm_stuttgart.mi.sd1;


import java.io.File;
import java.io.FileNotFoundException;
import java.io.PrintStream;

/**
 * Writing text lines to a file of given name
 */

public class Text2FileFlushable implements MyFlushable {

  private PrintStream out;

  /**
   * Prepare for writing strings to a file.
   *
   * @param fileName Calling {@link #println(String)} will add content to this file.
   * @throws FileNotFoundException Problem opening file i.e. inexistent parent directory.
   */
  public Text2FileFlushable(final String fileName)
     throws FileNotFoundException {
    out = new PrintStream(new File(fileName));
  }

  /**
   * Append to filename being defined in {@link #Text2File(String)} adding "\n".
   *
   * @param s String will be appended to file. Precondition: {@link #close()} ()}
   *          has not been called.
   */
  public void println(final String s) {
    out.println(s);
  }

  /**
   * Flushing pending output to underlying file.
   */
  public void flush(){
    out.flush();
  }
  /**
   * Closing file thereby flushing buffer. Caution: Further calls
   * to {@link #println(String)} will fail!.
   */
  public void close() {
    out.close();
    out = null;
  }
}