package de.hdm_stuttgart.mi.sd1.direction;

/**
 * Representing eight compass directions.
 */
public enum Direction {

  N(   0, "north"),
  NE( 45, "north by east"),
  E(  90, "east"),
  SE(135, "south by east"),
  S( 180, "south"),
  SW(225, "south by west"),
  W( 270, "west"),
  NW(315, "north by west");

  /**
   * A direction specified by angle and informal description.
   *
   * @param degree A value ranging from 0 (N) to 315 (NW) mimic a compass rose.
   * @param fullName The usual informal name i.e. north by west.
   */
  Direction(final int degree, final String fullName) {
    this.degree = degree;
    this.fullName = fullName;
  }

  @Override
  public String toString() {
    return fullName + " (" + degree + "°)";
  }

  public final int degree;
  public final String fullName;
}