package de.hdm_stuttgart.sw1.klausur;

public class StringTransforms {

	/**
	 * Eine urbane Legende attestiert Chinesen das Problem der Aussprache des
	 * Buchstabens "r". Sie würden stattdessen ein "l" verwenden.
	 * 
	 * Die Methode ersetzt alle Buchstaben 'r' durch 'l' bzw. 'R' durch 'L'.
	 * 
	 * Beispiel:
	 * 
	 * "Der arme Richard" ==> "Del alme Lichald"
	 * 
	 * @param input
	 *  Eine beliebige, von null verschiedene Zeichenkette
	 *            
	 * @return
	 * 	Die Ersetzung der Zeichenkette gemäß {'r' ==> 'l', 'R' ==> 'L'}
	 */
	public static String chinesischerAkzent(final String input) {
		

		final StringBuffer ret = new StringBuffer();
		for (final char c : input.toCharArray()) {
			switch (c) {
			case 'r':
				ret.append('l');
				break;
			case 'R':
				ret.append('L');
				break;
			default:
				ret.append(c);
			}
		}
		return ret.toString();
	}

	/**
	 * Sprachwissenschaftler sind der Ansicht, dass auch nach dem Entfernen von
	 * Vokalen der Text lesbar bleibt. Beispiel: "Die Anden" ==> "D ndn"
	 * 
	 * @param input
	 *            Eine beliebige Zeichenkette oder null
	 * 
	 * @return null, falls Eingabe null, sonst der Eingabestring ohne Vokale {aeiouAEIOU}
	 */
	public static String entferneVokale(final String input) {

		if (null == input) {
			return null;
		} else {
			final StringBuffer ret = new StringBuffer();
			for (final char c : input.toCharArray()) {
				switch (c) {
				case 'a':
				case 'e':
				case 'i':
				case 'o':
				case 'u':
				case 'A':
				case 'E':
				case 'I':
				case 'O':
				case 'U':
					break;
				default:
					ret.append(c);
				}
			}
			return ret.toString();
		}
	}

	/**
	 * Umwandlung eines Wortes (ohne Leerzeichen) in Morsezeichen gemäß dem Morse Alphabet als
	 * Folge langer (-) und kurzer (.) durch Pausen (Leerzeichen) getrennter Töne:
	 * 
	 * 		A .-    D -..  G --.  J .--- M --
	 *		B -...  E .    H .... K -.-  N -.
	 *		C -.-.  F ..-. I ..   L .-.. 
	 *      Buchstaben O bis Z sowie Ziffern 0 bis 9 aus Platzgründen weggelassen. Diese brauchen
	 *      in der Klausur aus Zeitgründen nicht implementiert zu werden.
	 *  
	 *  Regeln:
	 *   -Kleinbuchstaben werden zunächst in Großbuchstaben umgewandelt
	 *   -Pro Buchstabe / Ziffer wird nach der Morse Zeichenfolge eine Pause (Leerzeichen) eingefügt.
	 *  
	 *  Beispiel (Leerzeichen beachten, auch am Ende des Morsecodes):
	 *   "Abend" ==> ".- -... . -. -.. "
	 * 
	 * @param input
	 * 		Das Klartextwort, welches durch Morsezeichen kodiert werden soll.
	 * @return
	 * 		Die Übersetzung in das Morsealphabet.
	 */
	public static String morsen(final String input) {

		final String capitalizedInput = input.toUpperCase();

		final StringBuffer ret = new StringBuffer();
		for (final char c : capitalizedInput.toCharArray()) {
			switch (c) {

			case 'A':ret.append(".-");    break;
			case 'B':ret.append("-...");  break;
			case 'C':ret.append("-.-.");  break;
			case 'D':ret.append("-..");   break;
			case 'E':ret.append(".");     break;
			case 'F':ret.append("..-.");  break;
			case 'G':ret.append("--.");   break;
			case 'H':ret.append("....");  break;
			case 'I':ret.append("..");    break;
			case 'J':ret.append(".---");  break;
			case 'K':ret.append("-.-");   break;
			case 'L':ret.append(".-..");  break;
			case 'M':ret.append("--");    break;
			case 'N':ret.append("-.");    break;
			
// Zu viel Tipparbeit in einer Klausur
//			
//			case 'O':ret.append("---");   break;
//			case 'P':ret.append(".--.");  break;
//			case 'Q':ret.append("--.-");  break;
//			case 'R':ret.append(".-.");   break;
//			case 'S':ret.append("...");   break;
//			case 'T':ret.append("-");     break;
//			case 'U':ret.append("..-");   break;
//			case 'V':ret.append("...-");  break;
//			case 'W':ret.append(".--");   break;
//			case 'X':ret.append("-..-");  break;
//			case 'Y':ret.append("-.--");  break;
//			case 'Z':ret.append("--..");  break;
//
//			case '0':ret.append("-----"); break;
//			case '1':ret.append(".----"); break;
//			case '2':ret.append("..---"); break;
//			case '3':ret.append("...--"); break;
//			case '4':ret.append("....-"); break;
//			case '5':ret.append("....."); break;
//			case '6':ret.append("-...."); break;
//			case '7':ret.append("--..."); break;
//			case '8':ret.append("---.."); break;
//			case '9':ret.append("----."); break;

			default:
				throw new IllegalMorseCharacterException("Eingabezeichenkette '" + input + 
						"' enthält nicht-Morsezeichen '" + c + "'.");
			}
			ret.append(' ');
		}
		return ret.toString();
	}

}
