package de.hdm_stuttgart.sw1.klausur;


/**
 * Hilfsmethoden.
 *
 */
public class Helper {
   
   /**
    * <p>Liefert die Deutschen Klartextnamen der Zahlen von 0 bis Zwölf, also:</p>
    * 
    *  <p>{0, 1, 2, ..., 12} wird zu {"null", "eins", "zwei", ..., "zwölf"}</p>
    *  
    *  <p>Für alle anderen Werte, wie z.B. »14«, soll "Uuups, keine Regel
    *     für den Wert 14" zurückgegeben werden.</p>
    * 
    * @param a Umzuwandelnder Wert
    * @return Klartextname.
    */
   static public String klartextName(final int a) {
      switch(a) {
      case 0: return "null";
      case 1: return "eins";
      case 2: return "zwei";
      case 3: return "drei";
      case 4: return "vier";
      case 5: return "fünf";
      case 6: return "sechs";
      case 7: return "sieben";
      case 8: return "acht";
      case 9: return "neun";
      case 10: return "zehn";
      case 11: return "elf";
      case 12: return "zwölf";

      default : return "Uuups, keine Regel für den Wert " + a;
      }
   }

   /**
    * <p>Gibt den Betrag des Produkts der beiden Faktoren zurück.</p> 
    * 
    * @param a Erster Wert
    * @param b Zweiter Wert
    * @return Betrag des Produkts.
    */
   static public int getProduktBetrag(final int a, final int b) {
      return Math.abs(a * b);
   }

   /**
    * <p>Der Quadrant für einen gegebenen Winkel.</p> 
    * 
    * <p>Der Vollkreis wird in 0 bis 360 Grad eingeteilt. Je nach Winkel liegt ein Vektor
    * im Quadrant 1, 2 3 oder 4:</p> 
    *
    * <pre>
    *              |
    * Quadrant 2   |  Quadrant 1
    *              |
    *              |
    *--------------+-------------
    *              |
    *              |
    * Quadrant 3   | Quadrant 4
    *              |</pre>
    *              
    *  <p>Beispiel: 45 Grad liegt im ersten Quadranten, 200 Grad im Dritten.</p>            
    *  
    * @param winkel Winkel des zu betrachtenden Vektors aus dem Intervall [0, 360[.
    * @return zugehöriger Quadrant.
    */
   static public int getQuadrant(final double winkel) {
      if (winkel < 90) {
         return 1;
      } else if (winkel < 180) {
         return 2;
      } else if (winkel < 270) {
         return 3;
      } else {
         return 4;
      }
   }
	
   /**
    * <p>Umwandlung von Sekundenangaben in das Format hh:mm:ss.</p> 
    * 
    * <p>Beispiel: 7290 Sekunden entsprechen 2 Stunden, einer Minute und 30 Sekunden, also "2:1:30"</p> 
    * 
    * @param sekunden Nicht-negative Sekundenangabe
    * @return Angabe in Stunden:Minuten:Sekunden
    */
   static public String getZeitangabe(final long sekunden) {
      final long stunden = sekunden / (60 * 60);
      long rest = sekunden  % (60 * 60);
      long minuten = rest / 60;
      rest = rest % 60;
      return stunden + ":" + minuten + ":" + rest;
   }
	
   /**
    * <p>Kehre die Reihenfolge der Werte eines Feldes um. Beispiel:</p>
    *  
    *  <p> {1, 2, 3, 4, 5} wird zu {5, 4, 3, 2, 1}</p>
    * 
    * @param werte Ein Feld aus Werten oder null (keine Aktion).
    */
   static public void spiegelung(int [] werte) {
      if (null != werte) {
         final int length = werte.length;
         for (int i = 0; i < length / 2; i++) {
            final int save = werte[i];
            werte[i] = werte[length - i - 1];
            werte[length - i - 1] = save;
         }
      }
   }
   
   /**
    * <p>Prüfe die Gültigkeit einer ISBN Nummer.</p>
    * 
    * <p>Eine gegebene ISBN Nummer ist gültig, wenn:</p>
    * 
    * <ul>
    *   <li>Sie aus 10 Ziffern besteht und ansonsten nur "-" Zeichen enthält</li>
    *   
    *   <li><p>Das Produkt 1 * d_1 + 2 * d_2 + ... + 10 * d_10 ein Vielfaches von 11 ist, wobei {d_10 d_9 ...,d_1} die Ziffern von rechts nach links darstellen</p>
    *   <p>Beispiel: Die ISBN Nummer 0-201-31452-5 ist gültig:</p>
    *   <p>Produkt von rechts nach links 1*5 + 2*2 + 3*5 + 4*4 + 5*1 + 6*3 + 7*1 + 8*0 + 9*2 + 10*0 = 88, also ein Vielfaches von 11.</p>
    *   
    *   <p>18-433-6928-3 ist hingegen ungültig. Ebenso 2234-223-12, da nur 9 Ziffern vorhanden sind.</p>
    *   
    *   </li>
    * </ul>
    * 
    * @param isbn Kandidat für eine gültige ISBN Nummer.
    * @return true or false
    * 
    * <p style="color: red;font-weight: bold;">Tip: Beachten Sie {@link Character#getNumericValue(char)} und {@link String#toCharArray()}.</p>
    * 
    */
   static public boolean istGueltig(final String isbn) {
      int ziffernAnzahl = 0;
      int produktsumme = 0;
      for (final char c: new StringBuffer(isbn).reverse().toString().toCharArray()) {
         if (Character.isDigit(c)) {
            ziffernAnzahl++;
            produktsumme += ziffernAnzahl * Character.getNumericValue(c);
         } else if (c != '-') {
            return false; //
         }
      }
      if (10 == ziffernAnzahl && 0 == produktsumme % 11) {
         return true;
      } else {
         return false;
      }
   }
}