package de.hdm_stuttgart.mi.sd1.aufgabe1;

/**
 * <p>Sind Zeichenketten gleich oder ineinander enthalten?</p>
 */
public class Enthalten {

   /**
    * Linke Zeichenkette ist in rechter Zeichenkette enthalten
    */
   static public final int LINKS_IN_RECHTS_ENTHALTEN = 1;
   /**
    * Rechte Zeichenkette ist in linker Zeichenkette enthalten
    */
   static public final int RECHTS_IN_LINKS_ENTHALTEN = 2;
   /**
    * Zeichenketten sind gleich
    */
   static public final int GLEICH = 3;
   /**
    * Die Zeichenketten sind nicht ineinander enthalten
    */
   static public final int NICHT_ENTHALTEN = 4;
   
   /**
    * <p>Ist (mindestens) eine der beiden Zeichenketten in der jeweils anderen
    * ohne Beachtung von Groß- und Kleinschreibung enthalten?</p>
    * 
    * <p>Es sind folgende Fälle möglich:</p>
    * 
    *  <ol>
    *    <li>Beide Zeichenketten sind gleich.</li>
    *    <li>Die linke Zeichenkette ist in der rechten enthalten.</li>
    *    <li>Die rechte Zeichenkette ist in der linken enthalten.</li>
    *    <li>Beide Zeichenketten sind nicht ineinander enthalten.</li>
    *  </ol>
    *
    *  <p>Beispiel zur Verwendung (»Fach«, kleingeschrieben »fach«, ist in »Einfacher« enthalten):</p>
    *  
    *  <blockquote><pre>{@code final String s1 = "Einfacher Test",
    *             s2 = "Fach";
    *
    * switch(Helper.istEnthalten(s1, s2)) {
    *   case Helper.RECHTS_IN_LINKS_ENTHALTEN: 
    *     System.out.println(s2 + " ist in " + s1 + " enthalten");
    *     break;
    *     ...
    * }
    *}</pre></blockquote>
    *
    *  <p>Für die Ermittlung des Ergebnisses wird die oben aufgeführte Reihenfolge verwendet: Wenn beide Zeichenketten gleich sind,
    *     so sind sie auch in beiden Richtungen ineinander enthalten. Der Wert {@link #GLEICH} wird also
    *     gegenüber {@link #RECHTS_IN_LINKS_ENTHALTEN} und {@link #LINKS_IN_RECHTS_ENTHALTEN} bevorzugt.</p> 
    *     
    * 
    * @param links Die linke, von <code>null</code> verschiedene Zeichenkette.
    * @param rechts Die rechte, von <code>null</code> verschiedene Zeichenkette.
    * @return <blockquote><p>Genau einen der Werte mit nachfolgend genannten Prioritäten:</p>
    *   <ol>
    *     <li><p>{@link #GLEICH}</p></li>
    *     <li>
    *       <ul>
    *         <li>{@link #LINKS_IN_RECHTS_ENTHALTEN}</li>
    *         <li>{@link #RECHTS_IN_LINKS_ENTHALTEN}</li>
    *         <li>{@link #NICHT_ENTHALTEN}</li>
    *       </ul>
    *     </li>  
    *   </ol></blockquote>
    * @throws NullPointerException falls mindestens eines der Argumente <code>links</code> oder <code>rechts</code> <code>null</code> ist.
    */
   
   public static int istEnthalten(final String links, final String rechts) {
      
      // Tip: Konvertieren Sie beide Strings einheitlich entweder zur Klein- oder zur Großschreibung
      //      In der java.lag.String Klasse gibt es dafür eine passende Methode.

      if (links == null || rechts == null) {
         throw new NullPointerException();
      } else {
         
         final String linksKlein = links.toLowerCase(),
               rechtsKlein = rechts.toLowerCase();
         
         if (linksKlein.equals(rechtsKlein)) {
            return GLEICH;
         } else if(rechtsKlein.contains(linksKlein)) {
            return LINKS_IN_RECHTS_ENTHALTEN;
         } else if (linksKlein.contains(rechtsKlein)) {
            return RECHTS_IN_LINKS_ENTHALTEN;
         } else {
            return NICHT_ENTHALTEN;
         }
      }
   }
}