package de.hdm_stuttgart.mi.sd1.test.aufgabe1;

import org.junit.Assert;
import org.junit.FixMethodOrder;
import org.junit.Test;
import org.junit.runners.MethodSorters;

import de.hdm_stuttgart.mi.exam.unitmarking.ExaminationTestDefaults;
import de.hdm_stuttgart.mi.exam.unitmarking.Marking;
import de.hdm_stuttgart.mi.sd1.aufgabe1.Enthalten;

/**
 * Testen von {@link Enthalten}.
 *
 */
@SuppressWarnings({"UnusedDeclaration", "static-method"})
@FixMethodOrder(MethodSorters.NAME_ASCENDING)
public class Test_Enthalten  extends ExaminationTestDefaults {

   /**
    * Die linke Zeichenkette ist in der Rechten enthalten. Diese sind aber nicht gleich.
    * 
    */
   @Test
   @Marking(points=2)
   public void test_100_inZweitemEnthalten() {
      Assert.assertEquals(Enthalten.LINKS_IN_RECHTS_ENTHALTEN, Enthalten.istEnthalten("Ist", "Das ist richtig"));
   }

   /**
    * Die rechte Zeichenkette ist in der linken enthalten. Diese sind aber nicht gleich.
    * 
    */
   @Test
   @Marking(points=2)
   public void test_110_inErstemEnthalten() {
      Assert.assertEquals(Enthalten.RECHTS_IN_LINKS_ENTHALTEN, Enthalten.istEnthalten("eins ZwEi drei", "zWei"));
   }

   /**
    * Bis auf Groß- und Kleinschreibung sind beide Zeichenketten gleich.
    */
   @Test
   @Marking(points=2)
   public void test_130_EqualsGrossKleinSchreibung() {
      Assert.assertEquals(Enthalten.GLEICH, Enthalten.istEnthalten("Test", "test"));
   }

   /**
    * Zwei unterschiedliche Zeichenketten, welche wechselseitig nicht ineinander enthalten sind.
    * 
    */
   @Test
   @Marking(points=2)
   public void test_140_Verschieden() {
      Assert.assertEquals(Enthalten.NICHT_ENTHALTEN, Enthalten.istEnthalten("Dieses", "und etwas anderes"));
   }

   /**
    * Wenn mindestens eine der beiden Zeichenketten null ist, muss
    * eine NullPointerException geworfen werden.
    */
   @Test
   public void test_150_ErwarteteNullPointerException() {

      final String keineExceptionMeldung = "Die erwartete " + NullPointerException.class.getName() +
            " wurde nicht geworfen.";
      
      { // Zweiter Wert null
         boolean exceptionGeworfen = false;
         try {
            Enthalten.istEnthalten("Test", null);
         } catch (@SuppressWarnings("unused") final NullPointerException e) {
            exceptionGeworfen = true;
         }
         Assert.assertTrue(keineExceptionMeldung, exceptionGeworfen);
      }

      { // Erster Wert null
         boolean exceptionGeworfen = false;
         try {
            Enthalten.istEnthalten(null, "Test");
         } catch (@SuppressWarnings("unused") final NullPointerException e) {
            exceptionGeworfen = true;
         }
         Assert.assertTrue(keineExceptionMeldung, exceptionGeworfen);
      }

      {  // Beide Werte null
         boolean exceptionGeworfen = false;
         try {
            Enthalten.istEnthalten(null, null);
         } catch (@SuppressWarnings("unused") final NullPointerException e) {
            exceptionGeworfen = true;
         }
         Assert.assertTrue(keineExceptionMeldung, exceptionGeworfen);
      }
   }
}
