package de.hdm_stuttgart.mi.sd1.test.aufgabe2;

import org.junit.Assert;
import org.junit.FixMethodOrder;
import org.junit.Test;
import org.junit.runners.MethodSorters;

import de.hdm_stuttgart.mi.exam.unitmarking.ExaminationTestDefaults;
import de.hdm_stuttgart.mi.exam.unitmarking.Marking;
import de.hdm_stuttgart.mi.sd1.aufgabe2.Konto;

/**
 * Teste Verhalten bei Kontoerzeugung, Zu- und Abbuchungen.
 *
 */
@SuppressWarnings({"UnusedDeclaration", "static-method"})
@FixMethodOrder(MethodSorters.NAME_ASCENDING)
public class Test_Konto extends ExaminationTestDefaults {
   
   /**
    * Korrekter Anfangskontostand und einzelne Zubuchung.
    */
   @Test
   @Marking(points=4)
   public void testZubuchenDefault() {
      final Konto konto = new Konto();
      Assert.assertEquals(0, konto.getKontostand());
      
      konto.zubuchen(2000);
      Assert.assertEquals(2000, konto.getKontostand());
   }
   
   /**
    * Anfangsstand negativ und sofortige Abbuchung unterhalb
    * von {@link Konto#DISPOLIMIT}.
    */
   @Test
   @Marking(points=6)
   public void testAbbuchenNegativ() {
      final Konto konto = new Konto(-300);
      Assert.assertEquals(-300, konto.getKontostand());
      
      Assert.assertTrue(konto.abbuchen(2000));
      Assert.assertEquals(-2300, konto.getKontostand());
   }
   /**
    * Nach 5 aufeinanderfolgenden erfolgreichen Abbuchungen ist
    * keine weitere Abbuchung möglich. Danach erfolgt eine
    * einzelne Zubuchung gefolgt von einer weiteren Abbuchung.
    */
   @Test
   @Marking(points=4)
   public void testVerweigereFolge5() {
      final Konto konto = new Konto();
      Assert.assertEquals(0, konto.getKontostand());
      
      // 1. Abbuchung in Folge
      Assert.assertTrue(konto.abbuchen(100));
      Assert.assertEquals(-100, konto.getKontostand());
      
      // 2. Abbuchung in Folge
      Assert.assertTrue(konto.abbuchen(100));
      Assert.assertEquals(-200, konto.getKontostand());
      
      // 3. Abbuchung in Folge
      Assert.assertTrue(konto.abbuchen(100));
      Assert.assertEquals(-300, konto.getKontostand());
      
      // 4. Abbuchung in Folge
      Assert.assertTrue(konto.abbuchen(100));
      Assert.assertEquals(-400, konto.getKontostand());
      
      // 5. Abbuchung in Folge
      Assert.assertTrue(konto.abbuchen(100));
      Assert.assertEquals(-500, konto.getKontostand());
      
      // 6. Abbuchung in Folge, sollte scheitern
      Assert.assertFalse(konto.abbuchen(100));
      Assert.assertEquals(-500, konto.getKontostand());
      
      // 7. Erfolglose Zubuchung (Wert muss positiv sein)
      Assert.assertFalse(konto.zubuchen(0));
      Assert.assertEquals(-500, konto.getKontostand());

      // 8. Erfolgreiche Zubuchung
      Assert.assertTrue(konto.zubuchen(1));
      Assert.assertEquals(-499, konto.getKontostand());

      // 9. Abbuchung, davor erfolgte eine Zubuchung
      Assert.assertTrue(konto.abbuchen(100));
      Assert.assertEquals(-599, konto.getKontostand());
   }

   /**
    * Abbuchungen dürfen nur für positive Werte erfolgen. Abbuchungen mit
    * negativen Werten scheitern und werden nicht erfasst.
    */
   @Test
   @Marking(points=2)
   public void testFolge5() {
      final Konto konto = new Konto();
      Assert.assertEquals(0, konto.getKontostand());
      
      // 1. erfolglose Abbuchung in Folge
      Assert.assertFalse(konto.abbuchen(0));
      Assert.assertEquals(0, konto.getKontostand());
      
      // 2. erfolglose Abbuchung in Folge
      Assert.assertFalse(konto.abbuchen(0));
      Assert.assertEquals(0, konto.getKontostand());
      
      // 3. erfolglose Abbuchung in Folge
      Assert.assertFalse(konto.abbuchen(0));
      Assert.assertEquals(0, konto.getKontostand());
      
      // 4. erfolglose Abbuchung in Folge
      Assert.assertFalse(konto.abbuchen(0));
      Assert.assertEquals(0, konto.getKontostand());
      
      // 5. erfolglose Abbuchung in Folge
      Assert.assertFalse(konto.abbuchen(0));
      Assert.assertEquals(0, konto.getKontostand());
      
      // Abbuchung nach 5 erfolglosen Abbuchungen.
      Assert.assertTrue(konto.abbuchen(100));
      Assert.assertEquals(-100, konto.getKontostand());
   }
   
   /**
    * Abbuchungen an der Grenze zu {@link Konto#DISPOLIMIT}.
    */
   @Test
   @Marking(points=4)
   public void testDispolimit() {
      final Konto konto = new Konto(-3000);
      Assert.assertEquals(-3000, konto.getKontostand());
      
      // 1. Abbuchung, sollte am Dispolimit scheitern
      Assert.assertFalse(konto.abbuchen(1));
      Assert.assertEquals(-3000, konto.getKontostand());
      
      // 2. Zubuchung
      Assert.assertTrue(konto.zubuchen(1));
      Assert.assertEquals(-2999, konto.getKontostand());
      
      // 3. Abbuchung
      Assert.assertTrue(konto.abbuchen(1));
      Assert.assertEquals(-3000, konto.getKontostand());
   }
}
