package de.hdm_stuttgart.mi.sd1.test.aufgabe2;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Modifier;

import org.junit.Assert;
import org.junit.FixMethodOrder;
import org.junit.Test;
import org.junit.runners.MethodSorters;

import de.hdm_stuttgart.mi.exam.unitmarking.ExaminationTestDefaults;
import de.hdm_stuttgart.mi.exam.unitmarking.Marking;
import de.hdm_stuttgart.mi.sd1.aufgabe2.Ersatzteil;
import de.hdm_stuttgart.mi.sd1.test.helper.ReflectClass;
import de.hdm_stuttgart.mi.sd1.test.helper.ReflectionHelper;

import static org.hamcrest.Matchers.is;
import static org.hamcrest.Matchers.equalTo;
import static org.hamcrest.Matchers.not;
import static org.hamcrest.Matchers.nullValue;
import static org.hamcrest.Matchers.greaterThan;



/**
 * Teste Ersatzteile.
 *
 */

@FixMethodOrder(MethodSorters.NAME_ASCENDING)
@SuppressWarnings({"UnusedDeclaration"})
public class Test_Ersatzteil extends ExaminationTestDefaults {
	
	private final ReflectionHelper rHelper = new ReflectionHelper();

   /**
    * Korrekte toString() Redefinition.
    */
   @Test
   @Marking(points = 2)
   public void test_010_toString() {
   		Assert.assertThat(
   				new Ersatzteil(123, "Holzbalken").toString(),
					is(equalTo("Holzbalken, Teilenummer:123")));
   }
   /**
    * Korrekte toString() Redefinition.
    */
   @Test
   @Marking(points = 1)
   public void test_015_toString() {
   		Assert.assertThat(
   				new Ersatzteil(123, null).toString(),
					is(equalTo("Bezeichnung unbekannt, Teilenummer:123")));
   }
	/**
	 * Objektgleichheit positiv, gleiche Bezeichnung
	 */
	@Test
	@Marking(points = 2)
	public void test_020_equal() {

		Assert.assertThat(
				"Keine (geeignete) Ersatzteil.equals() Methode vorhanden!  ",
				rHelper.ersatzteil_equals.getRecursiveErrorMsg(),
				nullValue());

		Assert.assertThat(
				new Ersatzteil(23, "Mutter"),
				is(equalTo(new Ersatzteil(23, "Mutter"))));
	}
	/**
	 * Objektgleichheit positiv, ungleiche Bezeichnung
	 */
	@Test
	@Marking(points = 2)
	public void test_030_equalBezeichnungAnders() {

		Assert.assertThat(
				"Keine (geeignete) Ersatzteil.equals() Methode vorhanden!  ",
				rHelper.ersatzteil_equals.getRecursiveErrorMsg(),
				nullValue()
				);

		Assert.assertThat(
				new Ersatzteil(23, "Schraube"),
				is(equalTo(new Ersatzteil(23, "Mutter"))));
	}
	/**
	 * Objektgleichheit negativ, gleiche Bezeichnung
	 */
	@Test
	@Marking(points = 1)
	public void test_040_verschieden() {
		Assert.assertThat(rHelper.ersatzteil_equals.getRecursiveErrorMsg(), nullValue());
		Assert.assertThat(rHelper.ersatzteil_equals.assertIsDefinedIn(Ersatzteil.class),nullValue());

		Assert.assertThat(new Ersatzteil(24, "Mutter"),
				not(is(equalTo(new Ersatzteil(23, "Mutter")))));

	}
	/**
	 * Objektgleichheit negativ, ungleiche Bezeichnung
	 */
	@Test
	@Marking(points = 2)
	public void test_050_verschiedenBezeichnung() {

		Assert.assertThat(rHelper.ersatzteil_equals.getRecursiveErrorMsg(), nullValue());
		Assert.assertThat(rHelper.ersatzteil_equals.assertIsDefinedIn(Ersatzteil.class), nullValue());
		Assert.assertThat(new Ersatzteil(24, "Schraube"),
				not(is(equalTo(new Ersatzteil(23, "Mutter")))));
	}
	/**
	 * Objektgleichheit negativ, Vergleich mit null
	 */
	@Test
	@Marking(points = 2)
	public void test_060_verschiedenNull() {
		Assert.assertThat(rHelper.ersatzteil_equals.getRecursiveErrorMsg(), nullValue());
		Assert.assertThat(rHelper.ersatzteil_equals.assertIsDefinedIn(Ersatzteil.class), nullValue());
		Assert.assertThat(null, not(equalTo(new Ersatzteil(23, "Mutter"))));
	}
	/**
	 * Objektgleichheit negativ, Vergleich mit String
	 */
	@Test
	@Marking(points = 1)
	public void test_070_verschiedenString() {

		Assert.assertThat(rHelper.ersatzteil_equals.getRecursiveErrorMsg(), nullValue());
		Assert.assertThat(rHelper.ersatzteil_equals.assertIsDefinedIn(Ersatzteil.class), nullValue());
		Assert.assertThat("irgendwas", not(equalTo(new Ersatzteil(23, "Mutter"))));
	}

   /**
    * Alle Ersatzteil Attribute sollen private sein.
    */
   @Test
   @Marking(points = 1)
   public void test_075_attrib_ersatzteil_private() {

     Assert.assertThat("Die Klassse 'Ersatzteil' hat kein Attribut",
         Ersatzteil.class.getDeclaredFields().length,
         greaterThan(0));

     final String nonPrivateAttribs = ReflectClass.getNonPrivateAttributes(Ersatzteil.class);

      Assert.assertThat(
          "Folgende Attribute in der Klasse 'Ersatzteil' sind nicht 'private':  " + nonPrivateAttribs,
          nonPrivateAttribs,
          is(nullValue()));
   }
	
	/**
	 * Definition class Reifen {...}
	 */
	@Test
	@Marking(points = 1)
	public void test_080_classReifen() {
	  Assert.assertThat(rHelper.reifenKlasse.msg, is(nullValue()));
	}
	
	/** modifier and method.getModifiers();

	 * Konstruktor Definition public Reifen (int, int){...}
	 */
	@Test
	@Marking(points = 1)
	public void test_200_reifenKonstruktor() {
   Assert.assertThat(rHelper.reifen_constructor_int_int.getRecursiveMsg(), is(nullValue()));
	}

	/**
	 * Methode Ersatzteil.equals() in Ersatzteil darf nicht redefinierbar sein
	 */
	@Test
	@Marking(points = 1)
	public void test_077_equalsNichtRedefinierbar() {

   Assert.assertThat(rHelper.ersatzteil_equals.getRecursiveErrorMsg(), is(nullValue()));
   Assert.assertThat(rHelper.ersatzteil_equals.assertIsDefinedIn(Ersatzteil.class), is(nullValue()));
   Assert.assertThat(
       "Methode Ersatzteil.equals() darf nicht redefinierbar sein",
       rHelper.ersatzteil_equals.assertModifierIsPresent(Modifier.FINAL), is(nullValue()));
	}

	/**
	 * Methode Reifen.toString() mit korrekter Signatur definiert
	 */
	@Test
	@Marking(points = 1)
	public void test_220_Reifen_toString() {
   Assert.assertThat(rHelper.reifen_toString.getRecursiveErrorMsg(), is(nullValue()));
   Assert.assertThat(rHelper.reifen_toString.assertIsDefinedIn("de.hdm_stuttgart.mi.sd1.aufgabe2.Reifen"), is(nullValue()));
   Assert.assertThat(rHelper.reifen_toString.assertModifierIsPresent(Modifier.PUBLIC), is(nullValue()));
	}

	/**
	 * Erzeugung Instanz Reifen(final int number) ,Pruefung toString()
	 */
	@Test
	@Marking(points = 2)
	public void test_400_konstruktorReifen() {
   Assert.assertThat(rHelper.reifenKlasse.msg, is(nullValue()));
   Assert.assertThat(rHelper.reifen_constructor_int_int.getRecursiveMsg(), is(nullValue()));

		Object reifen;
		try {
			reifen = rHelper.reifen_constructor_int_int.constructor.newInstance(1132, 160);
		} catch (InstantiationException | IllegalAccessException | IllegalArgumentException
				| InvocationTargetException e) {
			Assert.fail("Konnte 'new Reifen(1132, 160)' nicht aufrufen.");
			return;
		}
		Assert.assertThat("Objekt ist kein Ersatzteil", reifen instanceof Ersatzteil, is(true));
		try {
		  Assert.assertThat(
		      rHelper.reifen_toString.method.invoke(reifen),
        equalTo("Reifen, Teilenummer:1132, Maximalgeschwindigkeit:160 Kmh"));
		} catch (final IllegalAccessException | IllegalArgumentException | InvocationTargetException e) {
			Assert.fail("Konnte 'toString()' für 'new Reifen(1132, 160)' nicht aufrufen.");
		}
	}
}
