package de.hdm_stuttgart.mi.sd1.test.helper;

import java.lang.reflect.Method;


/**
 * Describing a method to be instantiated by its string name within a given class
 *
 */
public class ReflectMethod {
	
	/**
	 * Metainfo about a method being defined in a given class.
	 * 
	 * @param reflectClass Describing the class in question
	 * @param reflectMethodName The method's name.
	 * @param argumentMeta The method's arguments
	 */
	public ReflectMethod(final ReflectClass reflectClass, final String reflectMethodName, final Class<?>[] argumentMeta) {
		this.definingReflectClass = reflectClass;
		Method methodTmp = null;
		String tmpMsg = null;
		if (null == reflectClass.classZ) {
			tmpMsg = "Folgefehler: Klasse '" + reflectClass.className + "' nicht definiert"; 
		} else {
		try { 
			methodTmp = reflectClass.classZ.getMethod(reflectMethodName, argumentMeta);
		} catch (NoSuchMethodException | SecurityException e) {
			tmpMsg = "Keine Methode '" + reflectMethodName + "(" + ")' in Klasse '" + reflectClass.className + "' gefunden";
		}
		}
		method = methodTmp;
		msg = tmpMsg;
	}
	
	/**
	 * Ausgangsklasse, in welcher die Methode vorhanden ist.
	 */
	public final ReflectClass definingReflectClass;
	
	/**
	 * 
	 */
	public final Method method;
	private final String msg;
	
	
	/**
	 * Providing error message in case the searched method was not found
	 * @return Explanation if method was not found, null otherwise
	 */
	public String getRecursiveErrorMsg() {
		if (null == definingReflectClass.msg) {
			return msg;
		} else {
			return "Folgefehler: " + definingReflectClass.msg;
		}
	}
	
   /**
    * Assert whether the method is being defined in a given class.
    * 
    * @param expectedClass
    * @return null if defined in expected class, error message otherwise
    */
   public String assertIsDefinedIn(final String expectedClass) {
      final Class<?> definingClass = method.getDeclaringClass();
      
      if (expectedClass.equals(definingClass.getName())) {
         return null;
      } else {
         return "equals(...) ist in '" +  definingClass.getName() + "' definiert, aber nicht in '"+ expectedClass + "'";
      }     
   }
   
   /**
    * Assert whether the method is being defined in a given class.
    * 
    * @param expectedClass
    * @return null if defined in expected class, error message otherwise
    */
   public String assertIsDefinedIn(final Class<?> expectedClass) {
      final Class<?> definingClass = method.getDeclaringClass();
      
      if (expectedClass.equals(definingClass)) {
         return null;
      } else {
         return "equals(...) ist in '" +  definingClass.getName() + "' definiert, aber nicht in '"+ expectedClass.getName() + "'";
      }     
   }
   
   /**
    * Check for presence of mandatory modifier.
    * @param modifier
    * @return null if desired modifier is present, descriptive message otherwise.
    * 
    */
   public String assertModifierIsPresent(final int modifier) {
      
      if (0 == (modifier & method.getModifiers())) {
         return "Modifier '" + ReflectClass.getModifierString(modifier) + "' not present";
      } else {
         return null;
      }
   }
   /**
    * Check for presence of mandatory modifier.
    * @param modifier
    * @return null if desired modifier is present, descriptive message otherwise.
    * 
    */
   public String assertModifierIsPresent(final int modifier, final String msg) {
      
      if (0 == (modifier & method.getModifiers())) {
         return msg;
      } else {
         return null;
      }
   }
}