package de.hdm_stuttgart.mi.sd1.aufgabe1;

/**
 * Hilfsmethoden.
 *
 */
public class Helper {

  /**
   * <p>Notenberechnung: In aufsteigender Abhängigkeit von erreichten Punkten sollen folgende Noten vergeben werden:</p>
   *
   * <dl>
   *   <dt>Unter 50 Punkten</dt>
   *   <dd><code>"mangelhaft"</code></dd>
   *
   *   <dt>Ab 50 Punkten:</dt>
   *   <dd><code>"ausreichend"</code></dd>
   *
   *   <dt>Ab 65 Punkten:</dt>
   *   <dd><code>"befriedigend"</code></dd>
   *
   *   <dt>Ab 80 Punkten:</dt>
   *   <dd><code>"gut"</code></dd>
   *
   *   <dt>Ab 95 Punkten:</dt>
   *   <dd><code>"sehr gut"</code></dd>
   *
   * </dl>
   *
   * @param punkte In einer Prüfung erreichte Punkte.
   *
   * @return Die erreichte Note in Textform gemäß Beschreibung.
   */
  static public String berechneNote(int punkte) {

    if (95 <= punkte) {
      return "sehr gut";
    } else if (80 <= punkte) {
      return "gut";
    } else if (65 <= punkte) {
      return "befriedigend";
    } else if (50 <= punkte) {
      return "ausreichend";
    } else {
      return "mangelhaft";
    }
  }

  /**
   * <p>Zerlegung eines Wertes in ein Array aus Einer, Zehner, Hunderter usw. Anteilen. Beispiele:</p>
   * <dl>
   *   <dt><code>0</code>:</dt>
   *   <dd><code>int[] result = {0}</code></dd>
   *
   *   <dt><code>123</code>:</dt>
   *   <dd><code>int[] result = {3, 20, 100}</code></dd>
   *
   *   <dt><code>43090</code>:</dt>
   *   <dd><code>int[] result = {0, 90, 0, 3000, 40000}</code></dd>
   *
   * </dl>
   * @param wert Der zu zerlegende Wert
   * @return Das Array der Zerlegung in Einer, Zehner, Hunderter usw. Anteile.
   */
  static public long[] dezimalZerlegung(long wert) {

    final long[] zerlegung = new long[Long.toString(wert).length()];

    int index = 0;
    long produkt = 1;
    do  {
      zerlegung[index++] = (wert % 10) * produkt; // Abspalten nächste Position
      produkt *= 10;
      wert /= 10;  // Reduktion, z.B. 1234 --> 123
    } while (0 < wert);

    return zerlegung;
  }

  /**
   * <p>Vertausche zwei Werte eines Arrays an den Indexpositionen <code>position</code> und
   * <code>position + 1</code>. Beispiel:</p>
   *
   * <p><code>values = {1, 2, 3, 4, 5}</code> und <code>position = 2</code>. Dann werden die Werte 3 und 4 an den
   * Indexpositionen 2 und 3 vertauscht, das Ergebnis lautet <code>{1, 2, 4, 3, 5}</code>.</p>.
   *
   * Falls das Array und der Wert von <code>position</code> eine Vertauschung zulassen, ist der Rückgabewert
   * <code>true</code>, ansonsten <code>false</code>.
   *
   * @param werte Ein Array aus Werten oder <code>null</code>.
   * @param position <p>Der Index des Werts, welcher mit dem Wert an der nächsthöheren Indexposition vertauscht werden
   *                 soll.</p>
   *                 <p>Bei negativem oder sonst für eine Vertauschung unpassendem Wert erfolgt keine Veränderung, der
   *                 Rückgabewert ist in diesem Fall <code>false</code>.</p>
   *
   * @return <code>true</code>, falls das Array nicht null ist, mindestens zwei Werte enthält und eine Vertauschung
   * der Werte an den Indexpositionen <code>position</code> und <code>position + 1</code> erlaubt. Ansonsten
   * <code>false</code>.
   */
  static public boolean swap(final int[] werte, final int position) {
    if (null == werte || position < 0 || werte.length - 1 <= position) {
      return false;
    } else {
      final int tmp = werte[position + 1];
      werte[position + 1] = werte[position];
      werte[position] = tmp;
      return true;
    }
  }

  /**
   * <p>Umwandlung jedes Wortanfangs in einen Großbuchstaben. Beispiel:</p>
   *
   * <p> <code>"Ein schöner warmer Sommer steht an."</code> wird zu
   *    <code>"Ein Schöner Warmer Sommer Steht An."</code></p>
   *
   * @param input Eine Satz aus Worten mit gemischter Groß- und Kleinschreibung.
   * @return Der Satz mit Großschreibung aller Wortanfänge oder <code>null</code>,
   *         falls die Eingabe <code>null</code> ist.
   */
   static public String getWortanfangGross(final String input) {
     if (null == input|| 0 == input.length()) {
       return input;
     } else {
       final StringBuilder result = new StringBuilder();
       result.append(Character.toUpperCase(input.charAt(0)));
       for (int i = 1; i < input.length(); i++) {
         final char currentChar = input.charAt(i);
         if (Character.isWhitespace(input.charAt(i - 1))) {
           result.append(Character.toUpperCase(currentChar));
         } else {
           result.append(currentChar);
         }
       }
       return result.toString();
     }
   }
}
