package de.hdm_stuttgart.mi.sd1.aufgabe1;

/**
 * <p>Bewertungen für das Würfelspiel «Mäxchen» (Schwäbisch «Mäxle»). Gespielt wird mit zwei
 * Würfeln, jeweils mit Augenzahl 1 bis 6:</p>
 *
 * <object data="doc-files/maexle.png" type="image/png"></object>
 *
 * <p>Die Schreibweise «(2|5)» bedeutet die Augenzahlen «2» und «5» auf den beiden
 * Würfeln, wobei die Reihenfolge irrelevant ist. Neben solchen «normalen» Ereignissen gibt es folgende Sonderfälle:</p>
 *
 * <dl>
 *   <dt>Pasch:</dt>
 *   <dd>(1|1), (2|2),... (6|6), beide Würfel haben dieselbe Augenzahl.</dd>
 *
 *   <dt>«Mäxchen» bzw. «Mäxle»:</dt>
 *   <dd>(2|1) bzw. (1|2)</dd>
 *
 * </dl>
 */
public class Maexchen {

  /**
   * Haben beide Würfel eine identische Augenzahl, so handelt es sich um einen Pasch.
   *
   * @param wuerfel1 Die Augenzahl des ersten Würfels.
   * @param wuerfel2 Die Augenzahl des zweiten Würfels.
   * @return <code>true</code>, falls es sich um einen Pasch handelt, ansonsten <code>false</code>.
   */
  static public boolean istPasch(final int wuerfel1, final int wuerfel2) {
    return wuerfel1 == wuerfel2;
  }

  /**
   * Das Ereignis (1|2) bzw. (2|1) stellt ein »Mäxchen« dar.
   *
   * @param wuerfel1 Die Augenzahl des ersten Würfels.
   * @param wuerfel2 Die Augenzahl des zweiten Würfels.
   * @return <code>true</code>, falls es sich um ein »Mäxchen« handelt, ansonsten <code>false</code>.
   */
  static public boolean istMaexchen(final int wuerfel1, final int wuerfel2) {
    return (wuerfel1 == 1 && wuerfel2 == 2)
        || (wuerfel2 == 1 && wuerfel1 == 2);
  }

  /**
   * <p>Ereignisse wie (1|3), (5|2) mit unterschiedlicher Augenzahl außer (1|2) bzw. (2|1) sind keine Sonderfälle.</p>
   *
   * @param wuerfel1 Die Augenzahl des ersten Würfels.
   * @param wuerfel2 Die Augenzahl des zweiten Würfels.
   * @return <code>true</code>, falls es sich um keinen Sonderfall handelt, ansonsten <code>false</code>.
   */
  static public boolean istKeinSonderfall(final int wuerfel1, final int wuerfel2) {
    return !istPasch(wuerfel1, wuerfel2)
        && !istMaexchen(wuerfel1, wuerfel2);
  }
}