package de.hdm_stuttgart.mi.sd1.aufgabe2;

/**
 * Darstellung von Verkaufsartikeln
 */
public class Artikel {
  /**
   * Die Bezeichung eines Artikels, z.B. »Fußball«
   */
  public final String artikelBezeichnung;

  /**
   * Die Artikelnummer.
   */
  final int artikelNummer;

  /**
   * Preisangabe in Cent
   */
  public final int preis;

  /**
   * Konstruktor für Artikelinstanzen.
   *
   * @param artikelBezeichnung Siehe {@link #artikelBezeichnung}.
   * @param artikelNummer Siehe {@link #artikelNummer}.
   * @param preis Siehe {@link #preis}.
   */
  public Artikel(final String artikelBezeichnung, final int artikelNummer, final int preis) {
    this.artikelBezeichnung = artikelBezeichnung;
    this.artikelNummer = artikelNummer;
    this.preis = preis;
  }

  /**
   * Zwei Artikel sind genau dann gleich, wenn ihre {@link #artikelNummer} identisch ist.
   * @param o Das zu vergleichende Objekt.
   * @return <code>true</code> falls das zu vergleichende Objekt ein {@link Artikel} mit
   * identischem Wert für die {@link #artikelNummer} ist, sonst <code>false</code>.
   */
  @Override
  public boolean equals(Object o) {
    if (o instanceof Artikel) {
      final Artikel artikel = (Artikel) o;
      return artikelNummer == artikel.artikelNummer;
    } else {
      return false;
    }
  }

  /**
   * Berechnung im Einklang mit  {@link #equals(Object)}.
   *
   * @return Siehe {@link Object#hashCode()}
   */
  @Override
  public int hashCode() {

    /*
       Hinweis: Achten Sie bei der Implementierung auf die Qualität erzeugter Hash Werte: Beispielsweise ist eine
       konstante Hash Funktion zwar formal korrekt, führt aber ggf. zu schlechter Performance.

       Die Qualität Ihrer Implementierung wird getestet.
     */
    return artikelNummer;
  }

  /**
   * @return Objektdarstellung als String
   */
  @Override
  public String toString() {
    return artikelBezeichnung + ", Artikelnummer " + artikelNummer + " zu " + euroDarstellung() + "€";
  }

  private String euroDarstellung() {
    if (preis < 100) {
      return "0." + preis;
    } else {
      final StringBuilder ret = new StringBuilder(Integer.toString(preis));
      return ret.insert(ret.length() - 2, ".").toString();
    }
  }
}
