package de.hdm_stuttgart.mi.sd1.task2;

import de.hdm_stuttgart.mi.sd1.task2.money.Coin;

/**
 *
 * <p>Representing money change amounts given in Euro (€) cents by numbers of coins. As an example we
 * compose an amount of 924 cents (9€ and 24 cents) starting from numbers of coins:</p>
 *
 * <object data="doc-files/euros.svg" type="image/svg+xml" width="700" height="400"></object>
 *
 *<table class="goikTableDefaults">
 *  <tr>
 *    <th>Description</th>
 *    <th>Code</th>
 *    <th>Result</th>
 *  </tr>
 *  <tr>
 *    <td>
 *      <ul>
 *        <li><code style="color:red">4</code> x «two euro» coins ({@link Coin#EURO_2}).</li>
 *        <li><code style="color:red">1</code> x «one euro» coin ({@link Coin#EURO_1}).</li>
 *        <li><code style="color:red">1</code> x «twenty cent» coin ({@link Coin#CENT_20}).</li>
 *        <li><code style="color:red">2</code> x «two cent» coin ({@link Coin#CENT_02}).</li>
 *      </ul>
 *      <p>Amount: 924 cents</p>
 *    </td>
 *    <td>
 *      <pre> final ChangeAmount c = new ChangeAmount();
 *
 * c.{@link #setCoin(Coin, int) setCoin}({@link Coin#EURO_2}, <code style="color:red">4</code>);
 * c.{@link #setCoin(Coin, int) setCoin}({@link Coin#EURO_1}, <code style="color:red">1</code>);
 * c.{@link #setCoin(Coin, int) setCoin}({@link Coin#CENT_20}, <code style="color:red">1</code>);
 * c.{@link #setCoin(Coin, int) setCoin}({@link Coin#CENT_02}, <code style="color:red">2</code>);
 *
 * System.out.println("Amount: " + c.{@link #getAmount()});</pre>
 *     </td>
 *     <td>
 *       <pre>Amount: 924</pre>
 *     </td>
 *   </tr>
 * </table>
 *
 * <p>Conversely a given euro cent amount can be decomposed into a minimal number of coins:</p>
 *
 *<table class="goikTableDefaults">
 *  <tr>
 *    <th>Code</th>
 *    <th>Result</th>
 *  </tr>
 *  <tr>
 *    <td>
 *      <pre> final ChangeAmount c = new ChangeAmount();
 *
 * c.{@link #setAmount(int) setAmount}(924); // 9€ and 24 cent
 *
 * final int[] coinCounts = c.{@link #getCoins(Coin[]) getCoins}(new Coin[]{
 *    Coin.EURO_2,
 *    Coin.EURO_1,
 *    Coin.CENT_20,
 *    Coin.CENT_02});
 *
 * System.out.println({@link java.util.Arrays#toString(int[]) Arrays.toString}(coinCounts));</pre>
 *     </td>
 *    <td>
 *      <pre>[4, 1, 1, 2]</pre>
 *     </td>
 *
 *   </tr>
 * </table>
 *
 * <p id="minimalNumberOfCoins">Notice that [0, 9, 0, 42] also represents an amount of 942 cents. But the
 * number of coins would not be minimal.</p>
 *
 *
 * <section class = "implementationHints">
 *   <h3>Implementation hints:</h3>
 *
 *   <ol>
 *     <li>
 *       <p>All enum {@link Coin} instances are present in the (correctly) ordered array returned
 *       by {@link Coin#values()}. The {@link Enum#ordinal()} method will return each {@link Coin} instance's
 *       unique index:</p>
 *
 *       <table class="goikTableDefaults">
 *         <tr>
 *           <th>Code</th>
 *           <th>Result</th>
 *         </tr>
 *         <tr>
 *           <td>
 *             <pre> for (final Coin c: Coin.values()) {
 *   System.out.println(<span style="color:red">c</span> + ", index=" + <span
 *     style="color:DarkOrange">c.ordinal()</span>);}</pre>
 *           </td>
 *           <td>
 *             <pre> <span style="color:red">CENT_01</span>, index=<span style="color:DarkOrange">0</span>
 * <span style="color:red">CENT_02</span>, index=<span style="color:DarkOrange">1</span>
 * <span style="color:red">CENT_05</span>, index=<span style="color:DarkOrange">2</span>
 * <span style="color:red">CENT_10</span>, index=<span style="color:DarkOrange">3</span>
 * <span style="color:red">CENT_20</span>, index=<span style="color:DarkOrange">4</span>
 * <span style="color:red">CENT_50</span>, index=<span style="color:DarkOrange">5</span>
 * <span style="color:red">EURO_1</span>, index=<span style="color:DarkOrange">6</span>
 * <span style="color:red">EURO_2</span>, index=<span style="color:DarkOrange">7</span></pre>
 *           </td>
 *         </tr>
 *       </table>
 *     </li>
 *
 *     <li>
 *       <p>Use an int[] array of size 8 matching the number of <code>8 == Coin.values().length</code> coins.
 *       Each position represents the count of the respective coin. Example:</p>
 *       <table class="goikTableDefaults">
 *         <tr>
 *           <th>Code</th>
 *           <th>Result</th>
 *         </tr>
 *         <tr>
 *           <td>
 *             <pre> final int[] coinCounts = new int[Coin.values().length];
 *
 * coinCounts[Coin.CENT_20.ordinal()] = <span style="color:red">2</span>;
 * coinCounts[Coin.EURO_2.ordinal()] = <span style="color:DarkOrange">3</span>;
 *
 * System.out.println(Arrays.toString(coinCounts));</pre>
 *           </td>
 *           <td>
 *             <pre> [0, 0, 0, 0, <span style="color:red">2</span>, 0, 0, <span style="color:DarkOrange">3</span>]</pre>
 *           </td>
 *         </tr>
 *       </table>
 *
 *       <p>The example array containing two «twenty cent» and three «two euro» coins represents
 *       an overall amount of 640 cents.</p>
 *     </li>
 *   </ol>
 * </section>
 */
public class ChangeAmount {


  /**
   * Get current amount.
   *
   * Example: One «two euro» coin and two «twenty cent» coins amount to 240 cents.
   *
   * @return The amount of all coins in Euro cent.
   */
  public int getAmount() {
    return -1234; // TODO:Implement me correctly
  }

  /**
   * Decompose a given Euro cents amount into «optimal» change i.e. requiring a minimal number of coins. See
   * <a href="#minimalNumberOfCoins">above remark</a> regarding the word «minimal».
   *
   * Example: 544 cent will be represented by:
   * <ul>
   *   <li>2 x «two euro» / {@link Coin#EURO_2} coin.</li>
   *   <li>1 x «one euro» / {@link Coin#EURO_1} coin.</li>
   *   <li>2 x «twenty cent» / {@link Coin#CENT_20} coin.</li>
   *   <li>2 x «two cent» / {@link Coin#CENT_02} coin.</li>
   *   <li>The remaining four coin types not being required will be absent.</li>
   * </ul>
   *
   * @param amount An amount provided as Euro-cent.
   */
  public void setAmount(int amount) {
    // TODO:Implement me correctly
  }

  /**
   * Set a certain type of coins to a given number.
   *
   * Example: After calling <code>setCoin(Coin.CENT_20, 3)</code> the change will contain three
   * «twenty cent» coins. Example:
   *
   * <pre>{@code
   * final ChangeAmount c = new ChangeAmount();
   *
   * c.setCoin(Coin.CENT_02, 2); // Adding 2 x «two cent» = 4 cent
   * c.setCoin(Coin.CENT_20, 2); // Adding 2 x «twenty cent» = 40 cent.
   * c.setCoin(Coin.CENT_50, 1); // Adding 1 x «fifty cent».
   *
   * System.out.println(c); // 94 cent}</pre>
   *
   * @param coin The coin, e.g. «fifty cent» to be set: Either of {@link Coin#values()}
   * @param num  The desired number of coins.
   */
  public void setCoin(final Coin coin, final int num) {
    // TODO:Implement me correctly
  }

  /**
   * Get number of selected coins within change. Example:
   *
   * <pre>{@code
   * final ChangeAmount c = new ChangeAmount();
   *
   * c.setAmount(241); // 241 Cent consisting of:
   *                   //  2 x «two Euro»
   *                   //  2 x «twenty cent»
   *                   //  1 x «one cent»
   *
   * // Query instance for counts of dedicated coins
   * final Coin[] coins = {Coin.EURO_2, Coin.CENT_50, Coin.CENT_20};
   * final int[] numbers = c.getCoins(coins);// Expecting {1, 0, 2}
   * }</pre>
   *
   * <code>numbers</code> will contain <code>{1, 0, 2}</code> representing one «two euro», zero «euro» and
   * two «twenty cent» coins.
   *
   * @param coins A list of coin values.
   * @return The individual number of coins for each desired position.
   */
  public int[] getCoins(final Coin[] coins) {
    return null; // TODO:Implement me correctly
  }
}