package de.hdm_stuttgart.mi.sd1.test.task1;

import de.hdm_stuttgart.mi.exam.unitmarking.ExaminationTestDefaults;
import de.hdm_stuttgart.mi.exam.unitmarking.Marking;
import org.junit.Assert;
import org.junit.FixMethodOrder;
import org.junit.Test;
import org.junit.runners.MethodSorters;

import java.util.Arrays;

import static de.hdm_stuttgart.mi.sd1.task1.HelperArray.getAlphabeticalFollowers;
import static de.hdm_stuttgart.mi.sd1.task1.HelperArray.getLeaders;

@FixMethodOrder(MethodSorters.NAME_ASCENDING)
public class Test_110_HelperArray_getAlphabeticalFollowers extends ExaminationTestDefaults {

  private static void assertArrayEmpty(final String[] values){
    Assert.assertArrayEquals(
        new String[]{},
        values);
  }

  @Test @Marking(points = 4) public void test_400_testSpecialCases() {

    assertArrayEmpty(getAlphabeticalFollowers(null, null));

    assertArrayEmpty(getAlphabeticalFollowers(null, ""));

    assertArrayEmpty(getAlphabeticalFollowers(null, "garbage"));

    assertArrayEmpty(getAlphabeticalFollowers(new String[]{}, null));

    assertArrayEmpty(getAlphabeticalFollowers(new String[]{}, ""));

    assertArrayEmpty(getAlphabeticalFollowers(new String[]{}, "garbage"));

    assertArrayEmpty(getAlphabeticalFollowers(new String[]{"Some Value"}, null));
  }

  private void assertFilter(final String[] expected, final String[] values, final String reference) {

    final String [] valuesCopy = Arrays.copyOf(values, values.length);

    final String[] result = getAlphabeticalFollowers(values, reference);

    Assert.assertNotSame("filtered array must not be equal to input", result, values);

    Assert.assertArrayEquals(expected, result);

    Assert.assertArrayEquals("Error: Original array value modifications", valuesCopy, values);

  }

  @Test @Marking(points = 3) public void test_460_testNormal() {

    assertFilter(new String[]{"pi", "sigma", "pi"},
                 new String[]{"alpha", "pi", "delta", "sigma", "pi"}, "gamma");

    assertFilter(new String[]{"pi", "delta", "sigma", "pi"},
              new String[]{"alpha", "pi", "delta", "sigma", "pi"}, "alpha");

    assertFilter(new String[]{"pi", "delta", "sigma", "pi"},
              new String[]{"pi", "delta", "alpha", "sigma", "pi"}, "alpha");

    assertFilter(new String[]{"pi", "delta", "sigma", "pi"},
              new String[]{"pi", "delta", "sigma", "pi", "alpha"}, "alpha");
  }

  @Test @Marking(points = 3) public void test_480_testNull() {
    assertFilter(new String[]{"sigma"}, new String[]{"alpha", null, "delta", "sigma", null}, "gamma");
  }

  @Test @Marking(points = 2)  public void test_500_testEmpty() {
    assertArrayEmpty(getAlphabeticalFollowers(new String[]{}, "alpha"));

    assertArrayEmpty(getAlphabeticalFollowers(new String[]{"alpha", "pi", "delta", "pi"}, "theta"));
    
    assertArrayEmpty(getAlphabeticalFollowers(new String[]{""}, ""));
  }

  @Test @Marking(points = 2) public void test_520_testAll() {

    final String[] values = {"alpha", "pi", "delta", "pi"};

    assertFilter(values, values, "");
    assertFilter(values, values, "alph");
  }
}