package de.hdm_stuttgart.mi.sd1.intmax;

import java.util.Arrays;

import org.junit.Assert;
import org.junit.FixMethodOrder;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.DisableOnDebug;
import org.junit.rules.TestRule;
import org.junit.rules.Timeout;
import org.junit.runners.MethodSorters;

/**
 * Unit test array max values.
 */
@SuppressWarnings({"UnusedDeclaration", "javadoc"})
@FixMethodOrder(MethodSorters.NAME_ASCENDING)
public class MaxTest {


   @Rule // Kill test if lasting longer than 1000 milliseconds unless in debugging mode
   public final TestRule timeout = new DisableOnDebug(Timeout.millis(1000));

   @Test
   public void test01SingleValue() {
      final int[] v = {-3},
            vCopy = Arrays.copyOf(v, v.length);

      Assert.assertEquals(0, Max.getFirstMaxIndex(v));
      Assert.assertArrayEquals(new int[] {0}, Max.getMaxIndexValues(v));
      Assert.assertArrayEquals(vCopy, v); // Don't touch original!
   }

   @Test
   public void test02UniqueValues() {
      final int[] v = {-3, 4, 100, 7, 11, 20},
            vCopy = Arrays.copyOf(v, v.length);
      Assert.assertEquals(2, Max.getFirstMaxIndex(v));
      Assert.assertArrayEquals(new int[] {2}, Max.getMaxIndexValues(v));
      Assert.assertArrayEquals(vCopy, v); // Don't touch original!
   }

   @Test
   public void test03DuplicateStartValues() {
      final int[] v = {40, 40, -0, 1, 7, 1, 12},
            vCopy = Arrays.copyOf(v, v.length);
      Assert.assertEquals(0, Max.getFirstMaxIndex(v));
      Assert.assertArrayEquals(new int[] {0, 1}, Max.getMaxIndexValues(v));
      Assert.assertArrayEquals(vCopy, v); // Don't touch original!
   }

   @Test
   public void test04DuplicateEndValues() {
      final int[] v = {-4, -4, 3, 7, 20, 20, 20},
            vCopy = Arrays.copyOf(v, v.length);
      Assert.assertEquals(4, Max.getFirstMaxIndex(v));
      Assert.assertArrayEquals(new int[] {4, 5, 6}, Max.getMaxIndexValues(v));
      Assert.assertArrayEquals(vCopy, v); // Don't touch original!
   }

   @Test
   public void test05DuplicateCenterValues() {
      final int[] v = {-4, -4, 3, 20, 20, 20, -6, 3},
            vCopy = Arrays.copyOf(v, v.length);
      Assert.assertEquals(3, Max.getFirstMaxIndex(v));
      Assert.assertArrayEquals(new int[] {3, 4, 5}, Max.getMaxIndexValues(v));
      Assert.assertArrayEquals(vCopy, v); // Don't touch original!
   }

   @Test
   public void test06MultipleGroupsValues() {
      final int[] v = {20, 20, -4, -4, 3, 20, 20, 20, -6, 3, 20, 20},
            vCopy = Arrays.copyOf(v, v.length);
      Assert.assertEquals(0, Max.getFirstMaxIndex(v));
      Assert.assertArrayEquals(new int[] {0, 1, 5, 6, 7, 10, 11}, Max.getMaxIndexValues(v));
      Assert.assertArrayEquals(vCopy, v); // Don't touch original!
   }
   @Test
   public void test07null() {
      Assert.assertArrayEquals(new int[]{}, Max.getMaxIndexValues(null));
   }
   @Test
   public void test08empty() {
	      Assert.assertArrayEquals(new int[]{}, Max.getMaxIndexValues(new int[] {}));
   }
}
