package de.hdm_stuttgart.mi.sda1.reewriteprepared.rdbms;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

/**
 * Helper class implementing database access.
 *
 */
public class CustomerInsert {

   private static Logger log = LogManager.getLogger(CustomerInsert.class);
   
   final Connection connection;
   final PreparedStatement searchExistingCustomer, insertCustomer;
   
   @SuppressWarnings("javadoc")
   public CustomerInsert(final Connection connection) throws SQLException {
      this.connection = connection;
      connection.setAutoCommit(true);
      searchExistingCustomer = connection.prepareStatement("SELECT uidNumber FROM Customers WHERE uid = ?");
      insertCustomer = connection.prepareStatement("INSERT INTO Customers (uid, cname) VALUES (?, ?)");
   }
   
   /**
    * Trying to add a new user represented by the user's uid (e.g. "fred") and common name (e.g. "Fred Astaire")
    * to the current database. An insertion attempt will only happen if no database entry of the corresponding
    * uid value yet exists.
    * 
    * @param uid The new user's uid value like "fred"
    * @param cname The new user's common name like "Fred Astaire"
    * @return true if new user has been inserted, false if uid is already present or arguments are either null or inappropriate.
    * @throws SQLException
    * 
    * Precondition: The values of uid and cname must not be null and within the given database schema boundaries
    *   i.e. concerning string length.
    *   
    */
   public boolean conditionallyInserCustomer (final String uid, final String cname) throws SQLException {

      final Statement statement = connection.createStatement();
      
      final String searchexistingUserQuery =
            "SELECT uidNumber FROM Customers WHERE uid = '" + uid + "'";
      
      try {
         final ResultSet existingUser = statement.executeQuery(searchexistingUserQuery);
         if (existingUser.next()) {
            log.warn("uid = '" + uid + "' is already in database");
            return false; // Insertion error: An entry with this uid already exists
         }
      } catch (SQLException e) {
         log.error("Unable search for user '" + uid + "': "+ e.getLocalizedMessage());
         throw e;
      }
      
      final String insertCustomerStatement = 
               "INSERT INTO Customers (uid, cname) VALUES ('" + uid + "', '" + cname + "')";
      
      try {
         final int numInsertedDatasets = statement.executeUpdate(insertCustomerStatement);
         if (1 == numInsertedDatasets) {
            log.info("Successfully inserted uid = " + uid);
            return true;
         } else {
            log.error("We should not be here: No exeption for failed customer insert uid = " + uid);
            return false;
         }
      } catch (SQLException e) {
         log.error("Unable to insert new customer: '" + e.getLocalizedMessage());
         throw e;
      }
   }
}
