package de.hdm_stuttgart.mi.sda1.exam.movie_by_actor;


import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.jdom2.Document;
import org.jdom2.Element;
import org.jdom2.filter.Filters;
import org.jdom2.xpath.XPathExpression;
import org.jdom2.xpath.XPathFactory;

import de.hdm_stuttgart.mi.unitmarking.jdom.DomFilter;

public class Moviedb2MovieByActor implements DomFilter {

	static private final String SEARCH_BY_ID_KEY = "searchById";

	static { // Initialize static parameterized Xpath query object,
		     // see https://freedocs.mi.hdm-stuttgart.de/__slidedom.html#/sda1_dom_fig_domXpathVariables.

		final Map<String, Object> xpathVarsNamespacePrefix = new HashMap<>();

		xpathVarsNamespacePrefix.put(SEARCH_BY_ID_KEY, null /* default namespace */);

		searchMovieByActorId = XPathFactory.instance().compile(
				"/movieDb/movies/movie[cast/actor[@ref=$" + SEARCH_BY_ID_KEY + "]]",
				Filters.element(),
				xpathVarsNamespacePrefix);
	}

	static private final XPathExpression<Element> searchMovieByActorId;

	@Override
	public Document process(final Document input) {

		final Element html = new Element("html");
		final Document htmlDoc = new Document(html);

		final Element body = new Element("body");
		html.addContent(body);

		final Element dl = new Element("dl");
		body.addContent(new Element("h1")
				.addContent("List of actors among with their corresponding films"))
				.addContent(dl);

		input.getRootElement().getChild("persons").getChildren().forEach(
				p -> addFilmsToActor(dl, p)
		);

		return htmlDoc;
	}

	private void addFilmsToActor(final Element dl, final Element person) {

		searchMovieByActorId.setVariable(SEARCH_BY_ID_KEY, person.getAttributeValue("id"));

		final List<Element> movies = searchMovieByActorId.evaluate(person);

		if (0 < movies.size()) {
			final Element dt = new Element("dt"),
					dd = new Element("dd");

			dl.addContent(dt).addContent(dd);
			dt.addContent(person.getChild("name").getText());

			movies.stream()
					.map(t -> t.getChild("title").getText())
					.forEach(s -> dd.addContent(new Element("p").addContent(s)));
		}
	}
}