package de.hdm_stuttgart.mi.sda1.exam.movie_by_actor;

import org.junit.Test;

import de.hdm_stuttgart.mi.exam.unitmarking.Marking;
import de.hdm_stuttgart.mi.unitmarking.jdom.DomAssert;
import de.hdm_stuttgart.mi.unitmarking.jdom.DomFilterTest;

@SuppressWarnings({"UnusedDeclaration"})
public class TestMoviesByActor extends DomFilterTest {

	static {
		init("Schema/sampledata.xml", new Moviedb2MovieByActor(), ".highlight.html");
	}

	@Test @Marking(points = 4)
	public void testNumberOfActors() {
		DomAssert.assertNumberOfNodes("Three actors expected", xmlRootElement, "body/dl/dt", 3);
		
		DomAssert.assertSingleNodeContent(
				"First actor 'Andie MacDowell' expected"
				,xmlRootElement
				,"body/dl/dt[1]"
				,"dt", "Andie MacDowell");

		DomAssert.assertSingleNodeContent(
				"Second actor 'Matthew Modine' expected"
				,xmlRootElement
				,"body/dl/dt[2]"
				,"dt", "Matthew Modine");

		DomAssert.assertSingleNodeContent(
				"Third actor 'R. Lee Ermey' expected"
				,xmlRootElement
				,"body/dl/dt[3]"
				,"dt", "R. Lee Ermey");
}

	@Test @Marking(points = 4)
	public void testNumberOfMovies() {
		DomAssert.assertNumberOfNodes("4 movies expected as <dl><dd><p>...</p>...", xmlRootElement, "body/dl/dd/p", 4);
		DomAssert.assertNumberOfNodes("4 movie paragraphs exclusively expected as <dl><dd><p>...</p>...", xmlRootElement, "body//p", 4);
		
		DomAssert.assertSingleNodeContent(
				"First movie 'Short Cuts' expected"
				,xmlRootElement
				,"body/dl/dd[1]/p"
				,"p", "Short Cuts");

		DomAssert.assertSingleNodeContent(
				"Second movie 'Full Metal Jacket' expected"
				,xmlRootElement
				,"body/dl/dd[2]/p[1]"
				,"p", "Full Metal Jacket");

		DomAssert.assertSingleNodeContent(
				"Third movie 'Short Cuts' expected"
				,xmlRootElement
				,"body/dl/dd[2]/p[2]"
				,"p", "Short Cuts");

		DomAssert.assertSingleNodeContent(
				"Fourth movie 'Full Metal Jacket' expected"
				,xmlRootElement
				,"body/dl/dd[3]/p"
				,"p", "Full Metal Jacket");
}

	@Test @Marking(points = 4)
	public void testModinesMovies() {
		
		DomAssert.assertNumberOfNodes(
				"Matthew Modine participates in two movies expecting '<dd><p>...</p><p>...</p></dd>'"
				, xmlRootElement
				, "body/dl/dt[. ='Matthew Modine']/following-sibling::dd[1]/p"
				, 2);

		DomAssert.assertNumberOfNodes(
				"Matthew Modine participates in two movies expecting '<dd><p>...</p><p>...</p></dd>'"
				, xmlRootElement
				, "body/dl/dt[. ='Matthew Modine']/following-sibling::dd[1]/*" //No other nodes than <p> below <dd>
				, 2);

		DomAssert.assertSingleNodeContent(
				"Matthew Modine's first movie is  expected as '<dd><p>Full Metal Jacket</p><p>...</p></dd>'"
				,xmlRootElement
				,"body/dl/dt[. ='Matthew Modine']/following-sibling::dd[1]/*[1]"
				,"p", "Full Metal Jacket");

		DomAssert.assertSingleNodeContent(
				"Matthew Modine's second movie is expected as '<dd><p>...</p><p>Short Cuts</p></dd>'"
				,xmlRootElement
				, "body/dl/dt[. ='Matthew Modine']/following-sibling::dd[1]/*[2]"
				,"p", "Short Cuts");
	}
}
