package de.hdm_stuttgart.mi.sda1.exam.dataexport;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;

import de.hdm_stuttgart.mi.sda1.exam.dataexport.model.Department;
import de.hdm_stuttgart.mi.sda1.exam.dataexport.model.Employee;

/**
 * Exporting company data. Execution requires an existing database schema and
 * optionally sample data. See file Sql/sample.sql.
 */
public class EmployeeInfo {
   
   /**
    * This refers to a corresponding element {@code <persistence-unit name = "strategy_none" >} 
    * in META-INF/persistence.xml leaving the Database "as is".
    */
   static private final String PERSISTENCE_UNIT_NONE = "strategy_none";
   /**
    * @param args Unused
    */
   public static void main( String[] args ) {

      final EntityManagerFactory factory = Persistence.createEntityManagerFactory(PERSISTENCE_UNIT_NONE);
      final EntityManager em = factory.createEntityManager();
      
      // Selecting all employees
      final CriteriaBuilder cb = em.getCriteriaBuilder();
      final CriteriaQuery<Employee> cq = cb.createQuery(Employee.class);
      cq.from(Employee.class); // Set query root, don't forget me!

      em.getTransaction().begin();

      int index = 1;
      for (final Employee e: em.createQuery(cq).getResultList()) {
         System.out.println(index++ + ": " + getInfo(e));
      }
      
      em.getTransaction().commit();

      em.close();
      factory.close(); 
   }
   
   private static String getInfo(final Employee e) {
      final StringBuilder ret = new StringBuilder(e.getCommonName() + ", department " + e.getDepartment().getName());
      
      final Department rootDepartment = getRootDepartment(e.getDepartment());
      if (rootDepartment != e.getDepartment()) {
         ret.append(", Top level department ");
         ret.append(rootDepartment.getName());
      }
      return ret.toString();
   }

   private static Department getRootDepartment(Department d) {
      while (null != d.getParent()) {
         d = d.getParent();
      }
      return d;
   }
}