package de.hdm_stuttgart.mi.sda1.exam.dataexport;

import org.jdom2.Element;
import org.jdom2.output.Format;
import org.jdom2.output.XMLOutputter;

import java.io.IOException;
import java.sql.*;

/**
 * Exporting company data. Execution requires importing corresponding
 * schema and optionally sample data from Sql/sample.sql
 */
public class Db2Html {

  /**
   * @param args Unused
   * @throws SQLException Many things may fail ...
   */
  public static void main(String[] args) throws SQLException, IOException {
    final Connection conn = DriverManager.getConnection(
      "jdbc:mysql://localhost:3306/hdm",
      "hdmuser",
      "XYZ");

    final Element result = getHtml(conn, "Student");

    final Format outFormat = Format.getPrettyFormat(); // Set output formatting
    //outFormat.setTextMode(Format.TextMode.PRESERVE);

    final XMLOutputter printer = new XMLOutputter(outFormat);
    printer.output(result, System.out);
  }

  /**
   * <p>Creating HTML tabular data corresponding to a single relational table. Typical output containing both
   * data and meta-data:</p>
   * <pre>&lt;html&gt;
   *  &lt;head&gt;&lt;title&gt;Table Student data&lt;/title&gt;&lt;/head&gt;
   *  &lt;body&gt;
   *    &lt;table&gt;
   *      &lt;tr&gt;
   *        &lt;th&gt;id&lt;/th&gt;
   *        &lt;th&gt;name&lt;/th&gt;
   *        &lt;th&gt;studyCourse&lt;/th&gt;
   *        &lt;th&gt;gender&lt;/th&gt;
   *        &lt;th&gt;fullTime&lt;/th&gt;
   *      &lt;/tr&gt;
   *  ...
   *      &lt;tr&gt;
   *        &lt;td&gt;101&lt;/td&gt;
   *        &lt;td&gt;John&lt;/td&gt;
   *        &lt;td&gt;PMM&lt;/td&gt;
   *        &lt;td&gt;m&lt;/td&gt;
   *        &lt;td&gt;
   *        &lt;strong&gt;NULL&lt;/strong&gt;
   *        &lt;/td&gt;
   *      &lt;/tr&gt;
   *    ...
   *    &lt;/table&gt;
   *  &lt;/body&gt;
   *&lt;/html&gt;</pre>
   *
   * @param connection A database connection
   * @param tableName A relational table name: Its meta information and contained data will be
   *                  converted to HTML.
   * @return The generated HTML tree containing a table similar to the example.
   * @throws SQLException A lot of things my fail ...
   */
  public static Element getHtml(final Connection connection, final String tableName) throws SQLException {

    final Element html = new Element("html");

    final Element head = new Element("head");
    html.addContent(head);

    final Element title = new Element("title");
    head.addContent(title);
    title.addContent("Table ").addContent(tableName).addContent(" data");

    final Element body = new Element("body");
    html.addContent(body);

    final Element table = new Element("table");
    body.addContent(table);

      final Statement statement = connection.createStatement();
      final ResultSet data = statement.executeQuery("SELECT * FROM "+ tableName);
      final ResultSetMetaData metaData = data.getMetaData();

      { // Table header containg column names
        final Element tr = new Element("tr");
        table.addContent(tr);
        for (int i = 1; i <= metaData.getColumnCount(); i++) {
          final Element th = new Element("th");
          th.addContent(metaData.getColumnName(i));
          tr.addContent(th);
        }
      }

      { // Table data
        while (data.next()) {
          final Element tr = new Element("tr");
          table.addContent(tr);
          for (int i = 1; i <= metaData.getColumnCount(); i++) {
            final Element td = new Element("td");
            tr.addContent(td);

            final String value = data.getString(i);
            if (data.wasNull()) {
              final Element strong = new Element("strong");
              td.addContent(strong);
              strong.addContent("NULL");
            }
            td.addContent(value);
          }
        }
      }
    return html;
  }
}