package de.hdm_stuttgart.mi.sda1;

import org.jdom2.*;
import org.jdom2.input.SAXBuilder;

import java.io.File;
import java.io.IOException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.SQLException;

public class Book2Rdbms implements AutoCloseable {

  static private final String
      insertSectionSql =
      "INSERT INTO  Section (id, title, parent, revisionflag)" +
          "          VALUES (?,  ?,     ?,      ?);",

  insertParaSql =
      "INSERT INTO  Para (parent, para, revisionflag) \n" +
          "VALUES (       ?,      ?,    ?);";

  private Connection conn;

  private PreparedStatement insertSection, insertPara;

  private Element book;

  public Book2Rdbms() {
    try {
      conn = DriverManager.getConnection(
          "jdbc:postgresql://localhost/hdm", "hdmuser", "XYZ");
      insertSection = conn.prepareStatement(insertSectionSql);

      insertPara = conn.prepareStatement(insertParaSql);

    } catch (SQLException e) {
      System.err.println("Unable to establish connection: " + e);
      System.exit(1);
    }

    final SAXBuilder builder = new SAXBuilder();
    try {
      book = builder.build(new File("Schema/sampledata.xml")).getRootElement();
    } catch (JDOMException e) {
      e.printStackTrace();
      System.err.println("Unable to parse input: " + e);
      System.exit(1);
    } catch (IOException e) {
      e.printStackTrace();
      System.err.println("Unable to read input: " + e);
      System.exit(1);
    }
  }

  public void execute() {
    // Top level <section> elements do not have a parent --> null.
    book.getChildren("section").forEach(s -> handleSection(null, s));
  }

  private void handleSection(final String parentId, final Element section) {
    try {
      final String currentSectionId = section.getAttributeValue("id");

      insertSection.setString(1, currentSectionId);
      insertSection.setString(2, section.getChildText("title"));
      insertSection.setString(3, parentId);
      insertSection.setString(4, section.getAttributeValue("revisionflag"));

      insertSection.execute();

      // add <para> children of current section
      handleParaChildren(section);

      // recurse to <section> children
      section.getChildren("section").forEach(s -> handleSection(currentSectionId, s));
    } catch (final SQLException e) {
      e.printStackTrace();
      System.err.println("Unable to insert <section>: " + e);
      System.exit(1);
    }
  }


  public void handleParaChildren(final Element section) {

    final String currentSectionId = section.getAttributeValue("id");
    section.getChildren("para").forEach(
        p -> {
          try {
            insertPara.setString(1, currentSectionId);
            insertPara.setString(2, p.getValue());
            insertPara.setString(3, p.getAttributeValue("revisionflag"));

            insertPara.execute();
          } catch (SQLException e) {
            e.printStackTrace();
          }
        }
    );
  }

  @Override
  public void close() {
    try {
      conn.close();
    } catch (final SQLException e) {
      e.printStackTrace();
    }
  }
}