package de.hdm_stuttgart.mi.sda1;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.jdom2.*;
import org.jdom2.input.SAXBuilder;

import java.io.File;
import java.io.IOException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.SQLException;

public class Category2Rdbms implements AutoCloseable {

  static private final Logger log = LogManager.getLogger(Category2Rdbms.class);

  static private final String
      insertCategorySql =
      "INSERT INTO Category (id, title)" +
          "VALUES               (?,  ?)",

  insertArticleSql =
      "INSERT INTO  Article (orderNo, name, description, img, imgType) " +
          "VALUES           (?,       ?,    ?,           ?,   ?)",

  insertCategoryArticleSql =
      "INSERT INTO Category2Article (category, orderNo)" +
          "VALUES                   (?,        ?)";

  private Connection conn;

  private Element catalog;

  private PreparedStatement insertCategory, insertArticle, relateCategoryArticle;

  public Category2Rdbms() {
    try {
      conn = DriverManager.getConnection(
          "jdbc:postgresql://localhost/hdm", "hdmuser", "XYZ");
      insertCategory = conn.prepareStatement(insertCategorySql);
      insertArticle = conn.prepareStatement(insertArticleSql);
      relateCategoryArticle = conn.prepareStatement(insertCategoryArticleSql);
    } catch (final SQLException e) {
      log.error("Unable to establish connection, exiting", e);
      System.exit(1);
    }

    final SAXBuilder builder = new SAXBuilder();
    try {
      catalog = builder.build(new File("Schema/sampledata.xml")).getRootElement();
    } catch (final JDOMException e) {
      log.error("Unable to parse input, exiting", e);
      System.exit(1);
    } catch (final IOException e) {
      log.error("Unable to read input, exiting", e);
      System.exit(1);
    }
  }

  public void execute() {
    log.info("Parsing XML input file");
    catalog.getChild("categories").getChildren("category").forEach(this::handleCategory);
    catalog.getChild("articles").getChildren("article").forEach(this::handleArticle);
  }

  private void handleCategory(final Element category) {
    log.info("Inserting category ");
    final String
        id = category.getAttributeValue("id"),
        title = category.getValue();
    log.info("Category id = " + id);
    try {
      insertCategory.setString(1, id);
      insertCategory.setString(2, title);
      insertCategory.execute();
    } catch (final SQLException e) {
      log.error("Failed to insert category id = " + id + ": ", e);
    }
  }

  private void handleArticle(final Element article) {
    log.info("Inserting article");

    final int orderNo = Integer.parseInt(article.getAttributeValue("orderNo"));

    final Element imgElement = article.getChild("img");

    final String
        name = article.getChildText("name"),
        description = article.getChildText("description"),
        img = (null == imgElement) ? null : imgElement.getAttributeValue("src"),
        imgType = (null == imgElement) ? null : imgElement.getAttributeValue("type");

    log.info("article orderNo = " + orderNo);
    try {
      insertArticle.setInt(1, orderNo);
      insertArticle.setString(2, name);
      insertArticle.setString(3, description);
      insertArticle.setString(4, img);
      insertArticle.setString(5, imgType);
      insertArticle.execute();
    } catch (final SQLException e) {
      log.error("Failed to insert article orderNo = " + orderNo + ": ", e);
    }

    log.info("Relating article " + orderNo + " to categories");
    article.getChildren("cat").forEach(cat -> {
      final String category = cat.getValue();
      try {
        relateCategoryArticle.setString(1, category);
        relateCategoryArticle.setInt(2, orderNo);
        relateCategoryArticle.execute();
      } catch (final SQLException e) {
        log.error("Failed to relate article orderNo = " + orderNo + "to category " + category, e);
      }
    });
  }

  @Override
  public void close() {
    try {
      conn.close();
    } catch (final SQLException e) {
      e.printStackTrace();
    }
  }
}