package de.hdm_stuttgart.mi.marking;

import de.hdm_stuttgart.mi.marking.implement.MarkFrequencies;

/**
 * <p>Generating an overview of marks by command line.</p>
 */
class FrequencyDriver {

  /**
   * <p>Reading marks as command line arguments.</p>
   *
   * @param marksAsStrings User entered mark strings e.g. <code>{"1", "4", ...}</code>.
   *
   */
  public static void main( String[] marksAsStrings ) { // rather than just »args«

    if (0 == marksAsStrings.length) { // No values present ?
      System.out.println("Usage: marks <mark1 mark2 ...>");
    } else {
      // Converting mark strings array e.g. {"1", "4", ...} to integer array {1, 4, ...}
      //
      final int[] marks = new int[marksAsStrings.length];
      for (int i = 0; i < marksAsStrings.length; i++) {
        final String markString = marksAsStrings[i];
        try {
          marks[i] = Integer.valueOf(markString);
        } catch (final NumberFormatException ex) {
          System.err.println("Error: Unable to process '" + markString + "'");
          System.exit(1); // Unsuitable user input
        }
      }
      // Feed values to into instance.
      final MarkFrequencies markFrequencies = new MarkFrequencies(marks);

      // Create overview of marks:
      //
      System.out.println("Mark|frequency");
      System.out.println("----+---------");

      for (int i = 0; i < markFrequencies.frequencyByMark.length ; i++) {
        System.out.println("   " + (markFrequencies.lowestMark + i - 1) + "|" +
            markFrequencies.frequencyByMark[i]);
      }
    }
  }
}