package de.hdm_stuttgart.mi.sd1.shape.model;

public class Rectangle extends Shape {

  private double width, height;

  /**
   * Extending {@link Shape#Shape(double, double)} by adding parameters
   * width and height on top of center coordinates.
   *
   * @param x First center coordinate component
   * @param y Second center coordinate component
   * @param width The rectangle's width
   * @param height The rectangle's height
   */
  public Rectangle(double x, double y, double width, double height) {
    super(x, y);
    setWidth(width);
    setHeight(height);
  }

  @Override
  public boolean equals(final Object o) {
    if (o instanceof Rectangle) {
      final Rectangle other = (Rectangle) o;
      return super.equals(o) &&  width == other.width && height == other.height;
    } else {
      return false;
    }
  }

  @Override
  public int hashCode() {
    return super.hashCode () + (int) (width + 13 * height);
  }

  @Override
  public Shape scale(final double factor) {
    width  *= factor;
    height *= factor;
    return this;
  }

  @Override
  public String toString() {
    return "Rectangle " + super.toString() + ", width=" + width + ", height=" + height;
  }

  public double getArea() {
    return width * height;
  }

  public double getPerimeter() {
    return 2 * (width + height);
  }

  // Getter and setter methods for private attributes
  public double getWidth() { return width;}
  public void setWidth(double width) { this.width = width;}
  public double getHeight() { return height;}
  public void setHeight(double height) { this.height = height;}
}