package de.hdm_stuttgart.mi.sd1.shape.model;

/**
 * Base class of geometric primitives like {@link Rectangle} and {@link Circle}
 */
public abstract class Shape {

  private double x, y;

  /**
   * Creating a shape with given center coordinates
   *
   * @param x The shape's center coordinate's x component
   * @param y The shape's center coordinate's y component
   */
  Shape(final double x, final double y) {
    setX(x);
    setY(y);
  }

  @Override
  public boolean equals(final Object o) {
    if (o instanceof Shape) {
      final Shape other = (Shape) o;
      return x == other.x && y == other.y;
    } else {
      return false;
    }
  }

  @Override
  public int hashCode() {
    return (int) (x + 13 * y);
  }

  @Override
  public String toString() {
    return "(" + x + ',' + y + ')';
  }

  /**
   * @param factor Scale the current figure by this value.
   * @return The current object.
   */
  public abstract Shape scale(double factor);

  /**
   * The current shape's area.
   *
   * @return e.g. with * height in case of a rectangle
   */
  public abstract double getArea();

  /**
   * The current shape's perimeter.
   *
   * @return e.g. with 2 * (height + width) in case of a rectangle
   */
  public abstract double getPerimeter();

  public double getX() {
    return x;
  }

  public void setX(final double x) {
    this.x = x;
  }

  public double getY() {
    return y;
  }

  public void setY(final double y) {
    this.y = y;
  }

  /**
   * Translating a shape by a given vector (x,y).
   *
   * @param x The translation vector's x component
   * @param y The translation vector's y component
   * @return The current object.
   */
  public final Shape move(final double x, final double y) {
    setX(getX() + x);
    setY(getY() + y);
    return this;
  }
}