package de.hdm_stuttgart.mi.sd1.filepath;

import org.junit.Assert;
import org.junit.Assume;
import org.junit.Test;

/**
 * OS dependent tests see
 * https://www.thekua.com/atwork/2008/09/running-tests-on-a-specific-os-under-junit4/
 */
@SuppressWarnings({"UnusedDeclaration"})
public class MetaInfoWindowsTest {
   
   static private final boolean isWindows = System.getProperty("os.name").startsWith("Windows");
   
   @Test
   public void testAbsoluteWithoutExtension() {
       Assume.assumeTrue(isWindows);
       
       final FileMetaInfo fmi = new FileMetaInfo("F:\\software\\my\\archive");
       
       Assert.assertFalse(fmi.isRelative);
       Assert.assertEquals('F', fmi.drive);
       Assert.assertEquals("\\software\\my", fmi.path);
       Assert.assertEquals("archive", fmi.basename);
       Assert.assertEquals(null, fmi.extension);
   }

   @Test
   public void testAbsoluteWithExtension() {
       Assume.assumeTrue(isWindows);
       
       final FileMetaInfo fmi = new FileMetaInfo("F:\\subdir\\archive.zip");
       
       Assert.assertFalse(fmi.isRelative);
       Assert.assertEquals('F', fmi.drive);
       Assert.assertEquals("\\subdir", fmi.path);
       Assert.assertEquals("archive", fmi.basename);
       Assert.assertEquals("zip", fmi.extension);
   }

   @Test
   public void testRelativeWithoutExtension() {
       Assume.assumeTrue(isWindows);
       
       final FileMetaInfo fmi = new FileMetaInfo("..\\Desktop\\icon");
       
       Assert.assertTrue(fmi.isRelative);
       Assert.assertEquals(FileMetaInfo.noDrive, fmi.drive);
       Assert.assertEquals("..\\Desktop", fmi.path);
       Assert.assertEquals("icon", fmi.basename);
       Assert.assertEquals(null, fmi.extension);
   }

   @Test
   public void testRelativeWithExtension() {
       Assume.assumeTrue(isWindows);
       
       final FileMetaInfo fmi = new FileMetaInfo("..\\..\\Desktop\\icon.gif");
       
       Assert.assertTrue(fmi.isRelative);
       Assert.assertEquals(FileMetaInfo.noDrive, fmi.drive);
       Assert.assertEquals("..\\..\\Desktop", fmi.path);
       Assert.assertEquals("icon", fmi.basename);
       Assert.assertEquals("gif", fmi.extension);
   }
}
