package de.hdm_stuttgart.mi.sd1.main;

import de.hdm_stuttgart.mi.sd1.store.IntegerStore;

/**
 * Calculate prime numbers in a more inefficient way by
 * subsequently storing them without allowing prime
 * number "gaps".
 *
 */
public class PrimeNumbers {
  
  private static IntegerStore primes = new IntegerStore(1000);
  
  /**
   * @param args unused
   */
  public static void main(String[] args) {    
    
    final long startTime = System.currentTimeMillis();
    final long numberOfPrimes = findPrimeNumbers(10_000_000);
    final long endTime = System.currentTimeMillis();
    
    System.out.println("Found " + numberOfPrimes + " prime numbers within " +
          (endTime - startTime) / 1000. + " seconds.");

  }
  
  /**
   * Find all prime numbers up to a given limit.
   * 
   * @param limit Test all integers from [2, limit]
   * whether they are prime numbers or not.
   * @return The number of primes found within [2, limit].
   * <dl>
      <dt>Precondition:</dt>
      <dd>2 &lt;= limit</dd>
    </dl>
   */
  public static long findPrimeNumbers(long limit) {
    primes.addValue(2); // Add a seed: The first prime number
    for (long i = 3; i <= limit; i++) {
       isPrime(i);
    }
    return primes.getNumValues();
  }
  
  /**
   * Test, whether a given number is prime. 
   * @param candidate The number to be assessed
   * @return true if candidate is prime, false otherwise
   * <dl>
      <dt>Precondition:</dt>
      <dd>2 &lt;= candidate</dd>
    </dl>
   */
  public static boolean isPrime(final long candidate) {
    for (int i = 0;
        primes.getValue(i) * primes.getValue(i) <= candidate;
        i++) { // Just test up to square
      if (0 == candidate % primes.getValue(i)) {
        return false;
      }
    }
    primes.addValue(candidate);
    return true;
  }
}
