package de.hdm_stuttgart.mi.sd1.grep;

import java.io.ByteArrayOutputStream;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.PrintStream;

import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;


// Credits to http://stackoverflow.com/questions/1119385/junit-test-for-system-out-println
//
@SuppressWarnings({"UnusedDeclaration"})
public class SearchFileTest {
	private final ByteArrayOutputStream outContent = new ByteArrayOutputStream();

	@Before
	public void setUpStreams() {
		System.setOut(new PrintStream(outContent));
	}

	@After
	public void cleanUpStreams() {
		System.setOut(null);
		outContent.reset();
	}

	@Test
	public void searchSingleMatchTest() throws CommandLineException {
		new Grep(new String[]{"Red", "Testdata/input.txt"}).process();

		Assert.assertEquals("Red wine is tasty\n", outContent.toString());
	}

	@Test
	public void searchMultipleMatchTest() throws CommandLineException {
		new Grep(new String[]{"-i", "Red", "Testdata/input.txt"}).process();

		Assert.assertEquals(
				"Red wine is tasty\n" 
						+ "The red cross acts worldwide\n",
						outContent.toString());
	}

	@Test
	public void searchMultipleFilesTest() throws CommandLineException {
		new Grep(new String[]{"red", "Testdata/input.txt", "Testdata/inputSecond.txt"}).process();

		Assert.assertEquals(
				"Testdata/input.txt:The red cross acts worldwide\n"
						+ "Testdata/inputSecond.txt:Errors will show up in red.\n",
						outContent.toString());
	}

	@Test
	public void searchMultipleFilesIgnoreCaseTest() throws CommandLineException {
		new Grep(new String[]{"-i", "ReD", "Testdata/input.txt", "Testdata/inputSecond.txt"}).process();

		Assert.assertEquals(
				"Testdata/input.txt:Red wine is tasty\n"
						+ "Testdata/input.txt:The red cross acts worldwide\n"
						+ "Testdata/inputSecond.txt:Errors will show up in red.\n",
						outContent.toString());
	}

	@Test
	public void searchMultipleFilenamesOnlyTest() throws CommandLineException {
		new Grep(new String[]{"-l", "Red", "Testdata/input.txt", "Testdata/inputSecond.txt"}).process();

		Assert.assertEquals(
				"Testdata/input.txt\n",
				outContent.toString());
	}
	@Test
	public void searchMultipleFilenamesOnlyIgnoreCaseTest() throws CommandLineException {
		new Grep(new String[]{"-i", "-l", "Red", "Testdata/input.txt", "Testdata/inputSecond.txt"}).process();

		Assert.assertEquals(
				"Testdata/input.txt\n"
						+ "Testdata/inputSecond.txt\n",
						outContent.toString());
	}

	@Test
	public void pipeShowContentTest() throws CommandLineException {

		try (final FileInputStream is =  new FileInputStream("Testdata/input.txt")){
			System.setIn(is);
			new Grep(new String[]{"-i", "Red"}).process();
			Assert.assertEquals(
					"Red wine is tasty\n" 
                  + "The red cross acts worldwide\n",
							outContent.toString());

			System.setIn(null);
		} catch (final IOException e) {
			System.err.println("Unable to open file:" + e.getLocalizedMessage());
		}
	}
	@Test
	public void pipeJustExistenceTest() throws CommandLineException {

		try (final FileInputStream is =  new FileInputStream("Testdata/input.txt")){
			System.setIn(is);
			new Grep(new String[]{"-l", "Red"}).process();
			Assert.assertEquals(
					"(standard input)\n",
							outContent.toString());

			System.setIn(null);
		} catch (final IOException e) {
			System.err.println("Unable to open file:" + e.getLocalizedMessage());
		}
	}
}
