package de.hdm_stuttgart.mi.sd1.connectfour;

import static de.hdm_stuttgart.mi.sd1.tictactoe.Player.YOU;
import static de.hdm_stuttgart.mi.sd1.tictactoe.Player.COM;

import org.junit.Assert;
import org.junit.Test;

import de.hdm_stuttgart.mi.sd1.tictactoe.Board;
import de.hdm_stuttgart.mi.sd1.tictactoe.Player;
/**
 * Testing Tic-tac-toe.
 */
@SuppressWarnings({"UnusedDeclaration"})
public class ScoreTest {

  @Test
  public void testEmptyBoard() {

    final Board board = new Board(
        COM,                      // active player.

        new Player[] {
            null,  null,  null,   //  | |
            //                       -+-+-
            null,  null,  null,   //  | |
            //                       -+-+-
            null,  null,  null}); //  | |

    Assert.assertEquals(0, board.getScore());
  }

  @Test
  public void testDrawTwoEnd() {

    final Board board = new Board(
        YOU,                      // active player.

        new Player[] { 
            YOU,  COM,  COM,      // O|X|X
            //                       -+-+-
            COM,  YOU,   YOU,     // X|O|O 
            //                       -+-+-
            null, null, COM});    //  | |X

    Assert.assertEquals(0, board.getScore());
  }
  
  @Test
  public void testDraw1() {

    final Board board = new Board(
        COM,                      // active player.

        new Player[] {
            null, null, null,     //  | | 
            //                       -+-+-
            null, YOU, null,      //  |O|  
            //                       -+-+-
            YOU, null, COM});     // O| |X

    Assert.assertEquals(0, board.getScore());
  }


  @Test
  public void testDrawEnd() {

    final Board board = new Board(
        COM,                      // active player.

        new Player[] { 
            YOU,  COM,  COM,      // O|X|X
            //                       -+-+-
            COM,  YOU,   YOU,     // X|O|O 
            //                       -+-+-
            YOU,  null, COM});    // O| |X

    Assert.assertEquals(0, board.getScore());
  }
  
  @Test
  public void testWinOne1() {

    final Board board = new Board(
        YOU,                      // active player.

        new Player[] {
            null, null, null,     //  | | 
            //                       -+-+-
            null, YOU, null,      //  |O|  
            //                       -+-+-
            YOU,  COM, COM});     // O|X|X

    Assert.assertEquals(YOU.score, board.getScore());
  }
  

  
  @Test
  public void testWinTwoMoves() {

    final Board board = new Board(
        COM,                      // active player.

        new Player[] {
            null, YOU, COM,       //  |O|X
            //                       -+-+-
            COM,  YOU, YOU,       // X|O|O 
            //                       -+-+-
            null, COM, null});    //  |X|

    Assert.assertEquals(COM.score, board.getScore());
  }
  
  @Test
  public void testLooseNextMove() {

    final Board board = new Board(
        COM,                      // active player.

        new Player[] {
            null,  YOU, YOU,       //  |O|O
            //                        -+-+-
            null,  COM, YOU,       //  |X|O 
            //                        -+-+-
            COM,  null, null});    // X| |

    Assert.assertEquals(YOU.score, board.getScore());
  }
}
