package de.hdm_stuttgart.mi.sd1;

import org.junit.Assert;
import org.junit.Test;

/**
 * Unit test of {@link Timeperiod}.
 */
@SuppressWarnings({"UnusedDeclaration"})
public class TimeperiodTest {

    // Helper methods for real tests.
    static private void assertPeriodEqualImplement(final int expectedSeconds,
                          final int expectedMinutes,
                          final int expectedHours,
                          final int expectedDays,
                          final int expectedWeeks,
                          final String expectedToString,
                          final Timeperiod period) {
        Assert.assertEquals(expectedSeconds, period.seconds);
        Assert.assertEquals(expectedMinutes, period.minutes);
        Assert.assertEquals(expectedHours, period.hours);
        Assert.assertEquals(expectedDays, period.days);
        Assert.assertEquals(expectedWeeks, period.weeks);

        Assert.assertEquals(expectedToString, period.toString());

    }

    static private void assertPeriodEqual(final int expectedSeconds,
                          final int expectedMinutes,
                          final int expectedHours,
                          final int expectedDays,
                          final int expectedWeeks,
                          final String expectedToString,
                          final Timeperiod period) {

        // Testing period in question
        assertPeriodEqualImplement(
                expectedSeconds, expectedMinutes, expectedHours, expectedDays, expectedWeeks, expectedToString,
                period);

        // Also testing copy constructor
        assertPeriodEqualImplement(
                expectedSeconds, expectedMinutes, expectedHours, expectedDays, expectedWeeks, expectedToString,
                new Timeperiod(period));
    }

    /**
     * Test constructor zero seconds.
     */
        @Test
        public void testZero() {
            assertPeriodEqual(
                    0,0,0,0,0,
                    "0 seconds",
                    new Timeperiod(0));
        }

    @Test
    public void testMinute() {
        assertPeriodEqual(0,1,0,0,0,
                "1 minute and 0 seconds",
                new Timeperiod(60));
        assertPeriodEqual(50,0,0,0,0,
                "50 seconds",
                new Timeperiod(50));
        assertPeriodEqual(12,1,0,0,0,
                "1 minute and 12 seconds",
                new Timeperiod(72));
        assertPeriodEqual(2,5,0,0,0,
                "5 minutes and 2 seconds",
                new Timeperiod(302));
    }

    @Test
    public void testHour() {
        assertPeriodEqual(0,0,2,0,0,
                "2 hours, 0 minutes and 0 seconds",
                new Timeperiod(7200));
        assertPeriodEqual(59,59,0,0,0,
                "59 minutes and 59 seconds",
                new Timeperiod(3599));
        assertPeriodEqual(40,1,1,0,0,
                "1 hour, 1 minute and 40 seconds",
                new Timeperiod(3700));
    }

    @Test
    public void testVarious() {
        assertPeriodEqual(1,3,4,1,6,
                "6 weeks, 1 day, 4 hours, 3 minutes and 1 second",
                new Timeperiod(3729781));

        assertPeriodEqual(23,56,17,5,3,
                "3 weeks, 5 days, 17 hours, 56 minutes and 23 seconds",
                new Timeperiod(2310983));
    }
}
