package de.hdm_stuttgart.mi.sd1.wc;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;

/**
 * Reading a text file and writing all content
 * to standard output
 */
public class Wc {

  /**
   * @param fileNames counting lines, words and characters of these file names 
   */
  public static void main(String[] fileNames) {
    
    TextFileStatistics[] textHandler = null;
    
    if (0 == fileNames.length) { // Try to read from standard input
      try {
        textHandler = new TextFileStatistics[] { new TextFileStatistics(new BufferedReader(new InputStreamReader(System.in)), "")};
      } catch(IOException ex) {
        System.err.println("unable to read from standard input");
        System.exit(1);
      }
    } else { // Try to read from given set of filenames
      textHandler = new TextFileStatistics[fileNames.length];
      for (int i = 0; i < fileNames.length; i++) {
        final String filename = fileNames[i];
        try {
          textHandler[i] = new TextFileStatistics(new BufferedReader(new FileReader(filename)), filename);
        } catch (IOException e) {
          System.err.println("unable to read from file '" + filename + "'");
          System.exit(1);
        }
      }
    }
    // We're done reading and parsing input(s). 
    // Write results to standard output.
    final String outputFormat = "%3d  %4d %6d  %s %n"; 
                              // int  int int string
                              //                 newline
    for (TextFileStatistics th : textHandler) {
      System.out.format(outputFormat, th.numLines, th.numWords, th.numCharacters, th.filename);
    }
    if (1 < textHandler.length) { // Multiple input files?
      System.out.format(outputFormat,
        TextFileStatistics.getTotalNumLines() 
      , TextFileStatistics.getTotalNumWords() 
      , TextFileStatistics.getTotalNumCharacters()
      , "total");
    }
  }
}