package de.hdm_stuttgart.mi.sda1.alignimg;

import java.io.IOException;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;

import de.hdm_stuttgart.mi.sda1.alignimg.handler.Img;
import de.hdm_stuttgart.mi.sda1.alignimg.handler.SearchAlignContentHandler;
import de.hdm_stuttgart.mi.sda1.alignimg.handler.SaxErrorHandler;

/**
 * A simple SAX parser demo
 * 
 */
public class App {
   private static final Logger log = LogManager.getLogger(App.class);

   /**
    * @param args
    *           Unused
    * @throws SAXException
    * @throws ParserConfigurationException
    * @throws IOException
    */
   public static void main(String[] args) throws ParserConfigurationException,
         SAXException, IOException {

      log.info("Creating SAX parser factory");
      final SAXParserFactory saxPf = SAXParserFactory.newInstance();

      log.info("Creating SAX parser");
      final SAXParser saxParser = saxPf.newSAXParser();
      final XMLReader xmlReader = saxParser.getXMLReader();

      log.info("Enabling namespaces and namespace prefix processing");
      xmlReader.setFeature("http://xml.org/sax/features/namespaces", true);
      xmlReader.setFeature("http://xml.org/sax/features/namespace-prefixes", true);

      log.info("Registering content- and error handler instances");
      final SearchAlignContentHandler searchAlignContentHandler = 
    		  new SearchAlignContentHandler();
      xmlReader.setContentHandler(searchAlignContentHandler);
      xmlReader.setErrorHandler(new SaxErrorHandler());

      final String xmlDocumentInstanceFilename = "src/main/resources/testdata.xhtml";
      log.info("Start parsing file '" + xmlDocumentInstanceFilename + "'");
      xmlReader.parse(xmlDocumentInstanceFilename);
      
      searchAlignContentHandler.alignImages.forEach(
    		  img -> System.out.println("Found " + img));
   }
}
