package de.hdm_stuttgart.mi.sda1.imgesearch;

import java.io.IOException;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;

import de.hdm_stuttgart.mi.sda1.imgesearch.handler.SaxContentHandler;
import de.hdm_stuttgart.mi.sda1.imgesearch.handler.SaxErrorHandler;


/**
 * Counting inline and block level img elements in Xhtml
 * 
 */
public class ImageCount {
   
   private static final Logger log = LogManager.getLogger(ImageCount.class);

   final SAXParserFactory saxPf;
   final XMLReader xmlReader;

   final SaxContentHandler ch;

   public ImageCount() throws ParserConfigurationException, SAXException {
      saxPf = SAXParserFactory.newInstance();
      final SAXParser saxParser = saxPf.newSAXParser();
      xmlReader = saxParser.getXMLReader();

      ch = new SaxContentHandler();
      xmlReader.setContentHandler(ch);
      xmlReader.setErrorHandler(new SaxErrorHandler());
      
   }

   public void read(final String htmlInput) throws IOException, SAXException {
      log.info("Start parsing  '" + htmlInput + "'");
      xmlReader.parse(htmlInput);
   }
   public int getNumberOfBlockLevelImgElements() { return ch.getNumberOfBlockLevelImgElements();}

   public int getNumberOfInlineImgElements() { return ch.getNumberOfInlineImgElements();}


   /**
    * @param args Unused
    * @throws SAXException 
    * @throws ParserConfigurationException 
    * @throws IOException 
    */
   public static void main( String[] args ) throws ParserConfigurationException, SAXException, IOException {

      final ImageCount imgCount = new ImageCount();

      final String xhtmlInstanceFilename = "src/main/resources/example.xhtml";

      imgCount.read(xhtmlInstanceFilename);

      System.out.println("Document contains " + imgCount.getNumberOfBlockLevelImgElements() +
            " block level <img> elements.");

      System.out.println("Document contains " + imgCount.getNumberOfInlineImgElements() +
            " inline <img> elements.");
   }
}
