package de.hdm_stuttgart.sda1.insert;

import org.junit.*;
import org.junit.runners.MethodSorters;
import org.springframework.core.io.ClassPathResource;
import org.springframework.jdbc.datasource.init.ScriptUtils;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ResourceBundle;

/**
 * Testing insert operations.
 */

@FixMethodOrder(MethodSorters.NAME_ASCENDING) // Subsequent SQL inserts require fixed execution order.
@SuppressWarnings({"UnusedDeclaration"})
public class InsertTest {

  static private Connection conn;
  static private Statement stmt;

  /**
   * Init connection
   * @throws java.sql.SQLException Unable to establish connection.
   */
  @BeforeClass
  static public void initDatabase() throws SQLException {

    conn = DriverManager.getConnection(
      SimpleInsert.jdbcProperties.getString("jdbcurl"),
      SimpleInsert.jdbcProperties.getString("username"),
      SimpleInsert.jdbcProperties.getString("password"));
    ScriptUtils.executeSqlScript(conn, new ClassPathResource("schema.sql"));
    stmt = conn.createStatement();
  }

  @AfterClass
  static public void releaseDatabase() throws SQLException {
    conn.close();
  }

  @Test
  public void test_010_insertJill() throws SQLException {
    Assert.assertEquals(1, SimpleInsert.insertPerson(stmt, "Jill", "jill@programmer.org"));
  }

  @Test
  public void test_020_insertDave() throws SQLException {
    Assert.assertEquals(1, SimpleInsert.insertPerson(stmt, "Dave", "dave@genius.org"));
  }


  /**
   * Insert fails due to UNIQUE constraint on email value »jill@programmer.org«
   *
   */
  @Test (expected = SQLException.class) // Duplicate key
  public void test_030_insertJillAgain() throws SQLException {
    Assert.assertEquals(0, SimpleInsert.insertPerson(stmt, "Jill", "jill@programmer.org"));
  }

}
