package dom;

import java.io.IOException;
import java.io.PrintStream;

import org.jdom2.Element;
import org.jdom2.JDOMException;
import org.jdom2.input.SAXBuilder;

import sax.MySaxErrorHandler;


/**
 * This class may be used for transforming XML sources to
 * XHTML by using {@link dom.HtmlTree}. 
 *
 */

public class Article2Html {

   private final SAXBuilder builder = new SAXBuilder();
   private final HtmlTree htmlResult;

   /**
    * Instances of this class allow for reading XML catalogs and delegate
    * XHTML transformation to a {@link HtmlTree} object,
    * see {@link #process(String, PrintStream)}. 
    */
   public Article2Html() {

      builder.setErrorHandler(new MySaxErrorHandler(System.out));

      htmlResult = new HtmlTree("Available articles",
            new String[] {"Article Description", "Order Number" });
   }

   /** Read an Xml catalog instance and insert product names among with their order numbers
    *  into a HTML DOM. Then serialize the resulting HTML tree to a stream. 
    * 
    * @param 
    *   filename of the Xml source.
    * @param out
    *    The output stream for HTML serialization. 
    * @throws IOException In case filename cannot be opened
    * @throws JDOMException Parsing error
    * 
    */
   public void process(final String filename, final PrintStream out) throws JDOMException, IOException{

      final Element articleList = builder.build(getClass().getClassLoader().getResource(filename)).
            getRootElement();

      articleList.getChildren().forEach(htmlResult::appendItem);
      htmlResult.serialize(out);
   }
}