package dom;

import java.io.IOException;
import java.util.List;

import org.jdom2.Attribute;
import org.jdom2.Element;
import org.jdom2.JDOMException;
import org.jdom2.input.SAXBuilder;

import sax.MySaxErrorHandler;

/**
 * Listing elements of arbitrary XML documents
 *
 */

public class ListElements {
   
   int numberOfAttributes, numberOfElements;

   private final SAXBuilder builder = new SAXBuilder();

   /**
    * Instances of this class allow for reading XML documents
    * producing a list of output elements
    */
   public ListElements() {
      builder.setErrorHandler(new MySaxErrorHandler(System.err));
   }

   /** Read an Xml document providing a list of elements being
    * contained within.
    * 
    * @param 
    *   filename of the Xml source.

    * @throws IOException In case filename cannot be opened
    * @throws JDOMException Parsing error
    * 
    */
   public void process(final String filename) throws JDOMException, IOException {
      numberOfAttributes = numberOfElements = 0;
      final Element root = builder.build(getClass().getClassLoader().getResource(filename)).getRootElement();
      descendDepthFirstPreOrder(root, 0);
   }
   
   /**
    * @return The count of all attributes in current XML.
    */
   public int getNumberOfAttributes() {
      return numberOfAttributes;
   }

   /**
    * @return The count of all elements in current XML.
    */
   public int getNumberOfElements() {
      return numberOfElements;
   }

   void descendDepthFirstPreOrder(final Element current, final int nestingDepth) {
      
      numberOfElements++;
      System.out.print("Element " + current.getName() + ", nesting depth = " + nestingDepth);
      
      // Show attributes among with their values
      final List<Attribute> attribs = current.getAttributes(); 
      if (!attribs.isEmpty()) {
         System.out.print(", Attributes: ");
         current.getAttributes().stream().forEach(this::handleAttribute);
      }
      System.out.println();
      
      // continue recursive descent
      current.getChildren().stream().forEach(e -> descendDepthFirstPreOrder(e, nestingDepth + 1));
   }
   
    void handleAttribute(final Attribute att) {
      numberOfAttributes++;
      System.out.print(att.getName() + " = '" + att.getValue() + "' ");
   }
   
}