package dom;

import java.io.IOException;

import org.jdom2.Attribute;
import org.jdom2.Element;
import org.jdom2.JDOMException;
import org.jdom2.input.SAXBuilder;
import org.jdom2.output.Format;
import org.jdom2.output.XMLOutputter;

import sax.MySaxErrorHandler;

/**
 * Cleaning up pre-HTML5 documents
 *
 */

public class Html2Html {
   
   private final SAXBuilder builder = new SAXBuilder();

   /**
    * Instances of this class allow for reading XML documents
    * producing cleaned HTML code
    */
   public Html2Html() {
      builder.setErrorHandler(new MySaxErrorHandler(System.err));
   }

   /** Read an HTML document and writing a cleaned version to standard output.
    * 
    * @param 
    *   filename of the HTML source.
    *   
    * @throws IOException In case filename cannot be opened
    * @throws JDOMException Parsing error
    * 
    */
   public void process(final String filename) throws JDOMException, IOException {
      final Element html = builder.build(getClass().getClassLoader().getResource(filename)).getRootElement();
      
      traverse(html);
      
      final Format outFormat = Format.getRawFormat();

      // Serialize to console
      final XMLOutputter printer = new XMLOutputter(outFormat);
      
      try {
         printer.output(html.getDocument(), System.out);
      } catch (IOException e) {
         e.printStackTrace();
         System.exit(1);
      }  
   }
   
   void traverse(final Element current) {
      if (current.getName().equals("img")) {

         final Attribute alignAtt = current.getAttribute("align");
         if (null != alignAtt) { // current <img .../> does have an align attribute.
            current.removeAttribute(alignAtt);// Kill deprecated attribute.
            
            final String css;
            switch (alignAtt.getValue()) {
            
            case "top":    css = "vertical-align: top;"; break;
            case "middle": css = "vertical-align: middle;"; break;
            case "bottom": css = "vertical-align: bottom;"; break;
            
            case "left":   css = "float: left;"; break;
            case "right":  css = "float: right"; break;

            default:       css = null;
               System.err.println("No idea how to handle illegal align='" + alignAtt.getValue() + "'");
               break;
            
            }
            current.setAttribute("style", css);// replace by CSS.
         }
      } else { // No <img/>, continue descending
         current.getChildren().stream().forEach(this::traverse);
      }
   }
}