package de.hdm_stuttgart.mi.persistence.test;

import java.sql.SQLException;

import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Test;

import de.hdm_stuttgart.mi.persistence.PersistenceHandler;
import de.hdm_stuttgart.mi.persistence.PersistenceHandler.Msg;

/**
 * Opening and closing connections
 */
@SuppressWarnings({"UnusedDeclaration"})
public class ConnectionTest {
   
   final ViewMockup viewMockup = new ViewMockup();

   final PersistenceHandler ph = new PersistenceHandler(viewMockup);
   
   
   
   /**
    * Initialize database
    * 
    */
   @BeforeClass
   static public void init() {
      try {
         InitDatabase.initDatabase();
      } catch (SQLException e) {
         Assert.fail("Unable to initialize database:" + e.getLocalizedMessage());
         return;
      }
   }
   

   /**
    * Opening and closing a connection
    */
   @Test
   public void openCloseConnection() {

      // Initially we should be in disconnected state
      Assert.assertFalse(ph.isConnected());
      
      // Connecting
      viewMockup.setStatusDisconnected();// Should be changed by subsequent call.
      ph.connect("admin", "Hello!");
      Assert.assertTrue(viewMockup.connected);
      Assert.assertEquals(PersistenceHandler.Msg.CONNECT_SUCCESSFUL.msg, viewMockup.info);
      
      // Disconnecting
      ph.disconnect();
      
      Assert.assertFalse(viewMockup.connected);
      Assert.assertEquals(Msg.DISCONNECT_SUCCESSFUL.msg, viewMockup.info);
   }

   /**
    * Unsuccessful attempt due to wrong credentials
    */
   @Test
   public void testAuthFailure() {

      // Initially we should be in disconnected state
      Assert.assertFalse(ph.isConnected());
      
      // Connecting
      viewMockup.setStatusDisconnected();// Should be changed by subsequent call.
      ph.connect("admin", "wrongPassword");
      Assert.assertFalse(viewMockup.connected);
      Assert.assertEquals(PersistenceHandler.Msg.UNABLE_TO_AUTHENTICATE.msg, viewMockup.error);
   }
}
