package de.hdm_stuttgart.mi.persistence.test;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Statement;

import de.hdm_stuttgart.mi.persistence.DbProps;

/**
 * Create database schema or erase entry records
 */
public class InitDatabase {

   /**
    * Erase if required and re-create application related tables.
    * 
    * @throws SQLException
    */
   static public void initDatabase() throws SQLException {

      try (
            final Connection conn = DriverManager.getConnection(
                  DbProps.getString("PersistenceHandler.jdbcUrl"),
                  DbProps.getString("PersistenceHandler.username"),
                  DbProps.getString("PersistenceHandler.password"));

            final Statement stmt = conn.createStatement();
            ) {
         stmt.executeUpdate("DROP TABLE IF EXISTS Person");
         stmt.executeUpdate("CREATE TABLE Person (\n" +
               "name CHAR(20) NOT NULL," +
               "email CHAR(20) NOT NULL UNIQUE," +
               "login CHAR(20) PRIMARY KEY," +
               "password TEXT" +
               ");");

         stmt.executeUpdate(
               "INSERT INTO Person VALUES (\n" +
                     "'Master admin',\n" +
                     "'admin@company.com',\n" +
                     "'admin',\n" +
                     "'PSvQ1Q==$x1Q26YkLNCnigl0mfq5nDJp6iEGd0LAwbEqx5pyB6A0='\n)"
               );
         stmt.close();
         conn.close();
      }
   }
   
   /**
    * Erase existing records.
    * 
    * @throws SQLException
    */
   static public void cleanDatabase() throws SQLException {

      try (
            final Connection conn = DriverManager.getConnection(
                  DbProps.getString("PersistenceHandler.jdbcUrl"),
                  DbProps.getString("PersistenceHandler.username"),
                  DbProps.getString("PersistenceHandler.password"));

                  final Statement stmt = conn.createStatement();
            ) {
         stmt.executeUpdate("DELETE FROM Person");
         conn.close();
      }
   }
}
