package de.hdm_stuttgart.mi.javastreams;

/*
 * Loosely derived from https://docs.oracle.com/javase/tutorial/java/javaOO/lambdaexpressions.html.
 */

/**
 * Representing students among with examination marks.
 *
 */
public class Student {

   /** Male or female */
   public enum Sex {
      /** */
      MALE("male"), 
      /** */
      FEMALE("female");

      final String extern;

      private Sex(final String extern) {this.extern = extern;}

      @Override
      public String toString(){return extern;}
   }

   String name; 
   int mark;
   Sex gender;
   String emailAddress;

   Student(String name, int mark ,
         Sex gender, String email) {
      this.name = name;
      this.mark = mark;
      this.gender = gender;
      this.emailAddress = email;
   }  

   /** * @return An examination's result */
   public int getMark() {
      return mark;
   }

   /** @return A student's sex. */
   public Sex getSex() {
      return gender;
   }

   /** @return A student's name. */
   public String getName() {
      return name;
   }

   /** @return A student's e-mail. */
   public String getEmailAddress() {
      return emailAddress;
   }

   /**
    * Compare two students by their marks
    * @param a first student
    * @param b second student
    * @return positive if student b's mark is better than student a's,
    *         zero if marks are equal, negative if ...
    */
   public static int compareByMark(final Student a, final Student b) {
      return a.mark - b.mark;
   }

   @Override
   public String toString() {
      return getName() + "(" + gender + ", " + getEmailAddress() + ", mark=" + getMark() + ")";
   }
}