package de.hdm_stuttgart.mi.sudoku.model;

import java.util.Locale;

import com.vaadin.data.util.converter.Converter;

public class IntegerConverter implements Converter <String, Integer> {
	
	final SudokuBoard sudokuBoard;
	final int xCoord;
	final int yCoord;
	private static final long serialVersionUID = 4897340072093396209L;
	
	public IntegerConverter(final SudokuBoard sudokuBoard, final int yCoord, final int xCoord){
		this.sudokuBoard = sudokuBoard;
		this.yCoord = yCoord;
		this.xCoord = xCoord;
	}
	
	@Override
	public Integer convertToModel(String value,
			Class<? extends Integer> targetType, Locale locale)
			throws ConversionException {
		if (null == value || 0 == value.trim().length()) {
			return new Integer(0);
		} else {
			try {
				final int v = Integer.parseInt(value);
				if (1 <= v && v <= 10) {
					final String errMsg = sudokuBoard.checkValue(v, yCoord, xCoord);
					if (null == errMsg) {
						return Integer.parseInt(value);
					} else {
						throw new ConversionException(errMsg);
					}
				} else {
					throw new ConversionException("Must be 1,2,... or 9");
				}
			} catch (NumberFormatException e) {
				throw new ConversionException("Must be 1,2,... or 9");
			}
		}
	} 

	@Override
	public String convertToPresentation(Integer value,
			Class<? extends String> targetType, Locale locale)
			throws com.vaadin.data.util.converter.Converter.ConversionException {
		if (null == value || value == 0) {
			return "";
		} else {
			return value.toString();
		}
	}

	@Override
	public Class<Integer> getModelType() {
		return Integer.class;
	}

	@Override
	public Class<String> getPresentationType() {
		return String.class;
	}
}
