package dom.xpath;

import java.io.IOException;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;


/** Check Readability of a given URL
 *
 */
public class CheckUrl {
   static private final Logger log = LogManager.getLogger(CheckUrl.class);

  /** Check whether a given object like http://someserver.com/img.gif
   *  can be retrieved.
   *  
   * @param urlRef
   *  The URL to be checked for read access
   *  
   * @return An informative URL related status message or null e.g. if the protocol in question is unknown.
   */
  public static boolean checkReadability(final String urlRef) {
    try {
      final URL url = new URL(urlRef);
      
      switch(url.getProtocol()) {
      
      case "http":
      case "https":
         try {
            final HttpURLConnection huc =  (HttpURLConnection) url.openConnection();
            huc.setRequestMethod("GET");
            huc.setInstanceFollowRedirects(false);
            huc.connect();
            return 200 == huc.getResponseCode(); // ignore redirects
         } catch (final IOException e) {
            log.error("Unable to connect to " + urlRef, e);
         }
         break;        
         default:
            log.error("Protocol '" + url.getProtocol() + "' not yet implemented");
      }
      
    } catch (final MalformedURLException e) {
       final String msg = "Address '" + urlRef + "' is malformed";
      System.err.println(msg);
      log.info(msg, e);
    }
    return false;
  }
}
