package dom.xpath;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import org.jdom2.Document;
import org.jdom2.Element;
import org.jdom2.JDOMException;
import org.jdom2.Namespace;
import org.jdom2.filter.ElementFilter;
import org.jdom2.input.SAXBuilder;
import org.jdom2.xpath.XPathExpression;
import org.jdom2.xpath.XPathFactory;

/**
 * Checking internal link consistency
 */
class CheckLocalReferences {

  private final SAXBuilder builder = new SAXBuilder();

  static private final Namespace htmlNamespace =
    Namespace.getNamespace("h","http://www.w3.org/1999/xhtml");

  static private final String ID_VAR_KEY = "targetId";

  static private final XPathExpression<Element> searchLocalHref, searchTargetId;

  static {
    final XPathFactory xpf = XPathFactory.instance();

    searchLocalHref = xpf.compile(
      "//h:*[starts-with(@href, '#')]",
      new ElementFilter(), null, htmlNamespace);

    final Map<String, Object> xpathVarsNamespacePrefix = new HashMap<>();
    xpathVarsNamespacePrefix.put(ID_VAR_KEY, null); // Variable having no namespace prefix

    searchTargetId = xpf.compile(
      "//h:*[starts-with(@id, $" + ID_VAR_KEY + ")]", // //h:*[starts-with(@id, $targetId)]
      new ElementFilter(),
      xpathVarsNamespacePrefix,
      htmlNamespace);
  }

  private Document htmlInput;

  /**
   * Read a HTML document and search for images. The image filenames will be
   * written to standard out.
   *
   * @param htmlFilename The XHTML's filename to be examined.
   * @throws JDOMException General parsing fault
   * @throws IOException   file access problem
   */
  void process(final String htmlFilename) throws JDOMException, IOException {

    htmlInput = builder.build(htmlFilename);

    searchLocalHref.evaluate(htmlInput).stream().
      map(a -> a.getAttributeValue("href")).
      forEach(this::checkLocalTarget);
  }

  private void checkLocalTarget(final String href) {
    final String id = href.substring(1);// strip off '#'

    searchTargetId.setVariable(ID_VAR_KEY, id);

    final int targetCount = searchTargetId.evaluate(htmlInput).size();

    switch (targetCount) {
      case 0:
        System.out.println("Error: matching target id '" + id + "' not found");
        break;
      case 1:
        System.out.println("matching target id '" + id + "' found");
        break;
      default:
        System.out.println("Error: Found " + targetCount + " matching id's for '" + id + "'");
        break;
    }
  }
}