package de.hdm_stuttgart.mi.sda1.sql2catalog;

import java.io.IOException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.jdom2.Element;
import org.jdom2.output.Format;
import org.jdom2.output.XMLOutputter;

/**
 * Accessing relational data.
 * 
 */
public class Rdbms2Xml {
   
   static final Logger log = LogManager.getLogger(Rdbms2Xml.class);

   /**
    * @param args Unused
    */
   public static void main(String[] args) {
      
      // Register Driver
      final String sqlDriverClassName = Messages.getString("Xml2Rdbms.jdbcDriverName"); 
      try {
         Class.forName(sqlDriverClassName);
      } catch (final ClassNotFoundException e) {
         log.error(Messages.getString("Xml2Rdbms.errMsgUnableRegisterDriverClass") + 
               sqlDriverClassName + "'", e);
         System.exit(1);
      }
      openConnection();
   }
   
   static void openConnection() {
      try (final Connection conn = DriverManager.getConnection(
            Messages.getString("Xml2Rdbms.jdbcUrl"),
            Messages.getString("Xml2Rdbms.jdbcUser"), 
            Messages.getString("Xml2Rdbms.jdbcPasswd"))) {

         initRdbms(conn);
         
      } catch (final SQLException e) {
         log.error("Unable to open connection:" + e);
         System.exit(1);
      }
   }
   
   static void initRdbms(final Connection conn) {
      try {
         appendProducts(new RdbmsAccess(conn));
      } catch (final SQLException e) {
         log.error("Unable to init database access layer:" + e);
         System.exit(1);
      }
   }
   
   static void appendProducts(final RdbmsAccess rdbmsAccess) {
      final Element catalogElement = new Element("catalog");
      try {
         rdbmsAccess.appendProducts(catalogElement);
      } catch (final SQLException e) {
         log.error("Unable to read database:", e);
         System.exit(1);
      }
      // Write XML content to standard output
      XMLOutputter outputter = new XMLOutputter(Format.getPrettyFormat());
      try {
         outputter.output(catalogElement, System.out);
      } catch (final IOException e) {
         log.error("Unable to write xml output data:" + e);
         System.exit(1);
      }
   }
}
