package de.hdm_stuttgart.mi.sda2.jpa.university;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityTransaction;
import javax.persistence.Persistence;
import javax.xml.bind.JAXBException;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import de.hdm_stuttgart.mi.sda2.jpa.cd.domain.Address;
import de.hdm_stuttgart.mi.sda2.jpa.cd.domain.Lecture;
import de.hdm_stuttgart.mi.sda2.jpa.cd.domain.Student;
import de.hdm_stuttgart.mi.sda2.jpa.cd.domain.StudentLecture;

/**
 * A simple http://logging.apache.org/log4j/2.x demo, see file log4j2.xml for
 * configuration options.
 * 
 */
public class Driver {
   private static Logger log = LogManager.getLogger(Driver.class);

   /**
    * @param args Unused
    *
    * @throws JAXBException Parsing configuration may fail
    */
   public static void main(String[] args) throws JAXBException {

      final EntityManagerFactory factory = Persistence
            .createEntityManagerFactory("persistenceUnit");
      final EntityManager manager = factory.createEntityManager();

      final EntityTransaction tx = manager.getTransaction();

      final Student joe = new Student("Joe Blix", 12346),
            eve = new Student("Eve Gardener", 54321);
      
      // Adding E-Mails
      joe.getEmails().add("joe@test.com");
      joe.getEmails().add("joe@blix.org");
      
      // Adding addresses
      joe.getAddresses().add(new Address("Schulstrasse 4", "Bad Oyenhausen", "32547"));
      joe.getAddresses().add(new Address("Am Märzenbaum 55", "Gummersbach", "66883"));
      
      eve.getAddresses().add(new Address("Grüner Weg 6", "Friedberg", "61169"));
      eve.getAddresses().add(new Address("Grüner Weg 6", "Friedberg", "61169"));
      
      final Lecture db = new Lecture("Database systems", 11312);
      final StudentLecture joeDb = new StudentLecture(joe, db, 5);

      tx.begin();
      {
         // manager.persist(joe); // not required if cascading persist is
         // enabled
         // manager.persist(db);
         manager.persist(joeDb);
         manager.persist(eve);
      }
      tx.commit();

      tx.begin();
      {
         manager.refresh(joe);
         log.info("Lectures of Joe:");
         for (StudentLecture sl : joe.getLectures()) {
            log.info(sl.getLecture().getTitle());
         }
      }
      tx.commit();
      
      tx.begin();
      {
         log.info("Reloading Joe:");
         final Student joeRetrieved = manager.find(Student.class, joe.getId());
         log.info(joeRetrieved.getFullName());
         log.info("Retrieving Joe's adresses:" );
         for (final Address a: joeRetrieved.getAddresses()) {
            log.info(a);
         }
      }
      tx.commit();

//      tx.begin();
//      {
//         log.info("Deleting Joe from Database");
//         manager.remove(joe);
//      }
//      tx.commit();

   }

}
