package de.hdm_stuttgart.mi.sda2.usermanage.parse;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import de.hdm_stuttgart.mi.sda2.usermanage.rdbms.DatabaseObject;


public class Group implements DatabaseObject {
   
  private static Logger log = LogManager.getLogger(Group.class);
   
   public static final Map<String, Set<Group>> secondaryGroupsByUid = new HashMap<String, Set<Group>>();
   public static final Map<Integer, Group> groupsByGidnumber = new HashMap<Integer, Group>();
   
   public Group(final String groupLine, int row) 
    throws ParseError {
      
      log.debug("Parsing group file line " + row + ":" + groupLine);
      
      int csvIndex = 0;
      final String [] components = groupLine.split(":");
      log.debug("Found " + components.length + " CSV components");
      
      // row 1
      gid = components[csvIndex++];
      
      csvIndex++; // We ignore the second CSV entry ("x") in row 2
      
      // row 3 
      try {
         gidNumber = Integer.parseInt(components[csvIndex++]);
      } catch (NumberFormatException e) {
         
         throw new ParseError("is not an integer (group)", row, csvIndex, components[csvIndex]);
      }

      // Optional row 4
      if (4 == components.length) { // Secondary memberships
         for (String uid : components[csvIndex++].split(",")) {
            log.debug("Adding secondary user '" + uid + "' to group '" + gid + "'");
            Set<Group> secondaryGroups = secondaryGroupsByUid.get(uid);
            if (null == secondaryGroups) {
               secondaryGroups = new HashSet<Group>();
               secondaryGroupsByUid.put(uid, secondaryGroups);
            }
            secondaryGroups.add(this);
         }
      }
      groupsByGidnumber.put(gidNumber, this);
   }
   public final String gid;
   public final int gidNumber;
   private int databaseId = 0;

   public int getDatabaseId() {
      return databaseId;
   }
   public void setDatabaseId(int databaseId) {
      this.databaseId = databaseId;
   }
   @Override
   public int hashCode() {
      return gidNumber;
   }
   @Override
   public boolean equals(Object obj) {
      if (obj instanceof Group) {
         return gidNumber == ((Group)obj).gidNumber;
      }
      return super.equals(obj);
   }
}
