package de.hdm_stuttgart.mi.sd1.aufgabe1;

/**
 * <p>Kostenabrechnung der Bodenverlegung eines einzelnen, rechteckförmigen Raums.
 * Dabei gibt es folgende Anteile:</p>
 *
 * <dl>
 *   <dt>Flächenkosten:</dt>
 *   <dd>Das Produkt aus der Anzahl verlegter Quadratmeter und den Kosten des verwendeten Materials
 *      (Teppich, Parkett, Fliesen,...) pro Quadratmeter.</dd>
 *
 *   <dt>Kosten Sockelleisten:</dt>
 *   <dd>Die Gesamtlänge der am Rand des Bodens verlegten Sockelleisten multipliziert mit dem
 *   Preis pro Meter der verwendeten Leiste.</dd>
 *
 *   <dt>Anfahrtskosten:</dt>
 *   <dd>
 *     <ul>
 *       <li>Pro Kilometer Anfahrt werden {@link #kostenProKilometer} berechnet.</li>
 *
 *       <li>Ab einem Grundpreis (Kosten ohne Anfahrt) von {@link #freigrenzeAnfahrt} werden keine
 *       Anfahrtskosten berechnet.</li>
 *
 *     </ul>
 *   </dd>
 *
 * </dl>
 *
 */
public class BodenlegerKosten {

  /**
   * Kosten pro Kilometer Anfahrtsweg in Euro.
   */
  public static final double kostenProKilometer = 0.3;

  /**
   * Ab diesem Wert für die eigentliche Verlegung entfallen die Kosten
   * für den Anfahrtsweg.
   */
  public static final double freigrenzeAnfahrt = 900.00;

  private final double breite, laenge;

  /**
   * Die Dimension des rechteckigen Raums.
   *
   * @param breite Raumbreite in Meter
   * @param laenge Raumlänge in Meter
   */
  public BodenlegerKosten(final double breite, final double laenge) {
    this.breite = breite;
    this.laenge = laenge;
  }

  /**
   * Raumfläche
   * @return Die Fläche des Raums in Quadratmetern.
   */
  public double getFlaeche() {
    return breite * laenge;
  }

  /**
   * Umfang, also Länge aller Seitenwände.
   * @return Der Umfang des Raums in Metern.
   */
  public double getUmfang() {
    return 2 * (breite + laenge);
  }

  /**
   * <p>Grundpreis für die Verlegung des Bodens und der Sockelleisten an den Rändern des Bodens.</p>
   *
   * <p>Beispiel: Für einen Raum der Größe 5m x 3m ergibt sich bei Laminat von 10€ pro Quadratmeter
   *  und Sockelleisten von 2€ pro Meter Länge:</p>
   *
   *  <p> 5 * 3 * 10€ + 2 * (5 + 3) * 2€</>
   *
   *  <p>Hinweis: Im Türbereich werden keine Sockelleisten benötigt. Dieser Unterschied
   *  wird als vernachlässigbar klein angesehen.</p>
   *
   * @param quadratmeterPreis Preis pro Quadratmeter des verwendeten Materials.
   * @param meterPreis Preis pro Meter der verwendeten Sockelleiste.
   *
   * @return Summe der Kosten verlegte Bodenfläche und Länge aller Sockelleisten.
   */
  public double getVerlegungskosten(final double quadratmeterPreis, final double meterPreis) {
    return breite * laenge * quadratmeterPreis +
      2 * (breite + laenge) * meterPreis;
  }

  /**
   * Gesamtkosten bestehend aus Verlegung und Anfahrtsweg.
   *
   * @param quadratmeterPreis
   * @param meterPreis
   * @param laengeAnfahrtsweg
   * @return Grundkosten gemäß {@link #getVerlegungskosten(double, double)} und Kosten für Anfahrtsweg
   * gemäß {@link #kostenProKilometer}, falls die Verlegungskosten kleiner {@link #freigrenzeAnfahrt} sind.
   */
  public double getGesamtKosten(final double quadratmeterPreis, final double meterPreis, final int laengeAnfahrtsweg) {

    final double verlegungsKosten = getVerlegungskosten(quadratmeterPreis, meterPreis);

    if (verlegungsKosten < freigrenzeAnfahrt) {
      return laengeAnfahrtsweg * kostenProKilometer + verlegungsKosten;
    } else {
      return verlegungsKosten;
    }
  }
}