Hibernate JPA 2 project template
================================

Usage:
------
    mvn archetype:generate -DarchetypeCatalog=local \
     -DarchetypeGroupId=de.hdm-stuttgart.mi \
     -DarchetypeArtifactId=mi-maven-archetype-jpa-hibernate \
     -DarchetypeVersion=1.2 \
     -DgroupId=de.mi.hdm_stuttgart -DartifactId=hellojpa \
     -DinteractiveMode=false

This requires to set up the MI Maven repository in `~/.m2/settings.xml`:

    <profiles>
      <profile>
        <id>hdm</id>
        <repositories>
          <repository>
            <id>archetype</id>
	        <url>https://maven.mi.hdm-stuttgart.de/Archetypes</url>
            <releases>
              <enabled>true</enabled>
              <checksumPolicy>warn</checksumPolicy>
            </releases>
            <snapshots>
              <enabled>true</enabled>
              <checksumPolicy>warn</checksumPolicy>
            </snapshots>
          </repository>
        </repositories>
      </profile>
    </profiles>


General:
--------
This project contains:

* `src/main/resources/META-INF/persistence.xml`

   Two persistence units offering a »drop-and-create«
   (re-creating schema) and a »strategy_none« (don't touch)JDBC database
   connection to a Mysql 5 server. You may want to adjust both database
   type and connection parameters.

* Executable classes and tests:

|Class          |Package               |Description           |
|---------------|----------------------|----------------------|
|`Airline`      |`$(packageroot).model`|JPA example entity    |
|`CreateAirline`|`$(packageroot)`      |Create database schema, persist an instance of Airline   |
|`SearchByIcao` |`$(packageroot)`      |JPA typesafe criteria search demo |
|`EntityTest`   |`$(testroot)`         |Persisting and re-reading an object by its primary key value |

* `src/main/resources/log4j2.xml` Configuring the log4j subsystem.

On executing i.e. »`mvn install`« Maven will
[generate]
(https://docs.jboss.org/hibernate/orm/current/topical/html_single/metamodelgen/MetamodelGenerator.html)
metamodel classes like `Airline_.java` during the
»`generate-sources`« phase. These classes are being required for
[typesafe criteria based queries]
(http://docs.jboss.org/hibernate/entitymanager/3.6/reference/en/html/querycriteria.html#querycriteria-typedquery)
as being outlined in class `$(packageroot)`/`SearchByIcao`.


Targets:
--------

* `mvn test`
  Creating and re-reading a database instance of `Airline`. Connection
  parameters being defined twice in
  `src/main/resources/META-INF/persistence.xml`.

* `mvn package`
  Creating an executable jar persisting `Airline` instances. To be executed like:

    `java -jar target/intro-1.0-SNAPSHOT.jar`

Note to Idea IDE users:
-----------------------
The `com.mysema.query.apt.jpa.JPAAnnotationProcessor` must be configured manually inside the IDE
although being defined in `pom.xml` accordingly:


![IDEA annotation processor configuration](Doc/ideaConfigAnnotationProcessing.png.marked.png)

1. Folder containing generated classes
2. First annotation processor being imported `from pom.xml`
3. Second processor to be configured manually: `com.mysema.query.apt.jpa.JPAAnnotationProcessor`

Note to Eclipse IDE users:
--------------------------
On initial archetype based project generation your new project's
class »`SearchByIcao`« reveals a compile time error indicating a
missing class `Airline_`. Possible resolutions:

* Right click on your project's root choosing
»`Run As`« ➩ »`Maven generate-sources`«

* Execute »`mvn generate-sources`« manually from the command line.

This step derives the missing metamodel class from `Airline`
thereby resolving the error.
When working with the Eclipse IDE (rather than with Maven directly)
this step is required whenever modifying your JPA data model.

IDE problems of this type are due to Eclipse's (and other IDE's)
limited Maven integration. The above workaroud triggers
the metamodel generator explicitely.
