#set( $symbol_pound = '#' )
#set( $symbol_dollar = '$' )
#set( $symbol_escape = '\' )
package ${package};

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Root;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import de.hdm_stuttgart.mi.sd1.model.Airline;
import de.hdm_stuttgart.mi.sd1.model.Airline_; // See comment further below.

public class CriteriaSearchByIcao {
    static private final Logger log = LogManager.getLogger(CriteriaSearchByIcao.class);

    /**
     * This refers to a corresponding element <persistence-unit name = "strategy_none" >
     * in META-INF/persistence.xml preserving the relational database schema.
     */
    static private final String PERSISTENCE_UNIT_NAME = "strategy_none";

    static private final EntityManagerFactory factory = Persistence.createEntityManagerFactory(PERSISTENCE_UNIT_NAME);
    static private final EntityManager entityManager = factory.createEntityManager();

    /**
     * @param args Unused
     */
    public static void main( String[] args ) {

        final CriteriaBuilder cb = factory.getCriteriaBuilder();

        log.info("Criteria search for Airline instances having icaoCode starting with 'D'");

        final CriteriaQuery<Airline> criteria = cb.createQuery( Airline.class );
        final Root<Airline> airlineRoot = criteria.from( Airline.class );

        // Eclipse users note: The subsequent line of code requires "mvn generate-sources" or equivalent
        // triggering the metamodel generator. You may want to right click on your project root choosing
        //
        // "Run As" --> "Maven generate-sources".
        //
        // This step is being required whenever modifying your model classes in de.hdm_stuttgart.mi.sd1.model.

        criteria.where(cb.like( airlineRoot.get( Airline_.icaoCode), "D%" ));

        entityManager.
                createQuery( criteria ).
                getResultList().
                stream().
                map(s -> "like 'D%': Read »" + s + "«").
                forEach(System.out::println);

        factory.close();
    }
}