package de.hdm_stuttgart.mi.exam.unitmarking;

import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map;

import org.junit.runner.Description;
import org.junit.runner.notification.Failure;
import org.junit.runner.notification.RunListener;

public class MarkingListener extends RunListener {
   
  static <T> T last(T[] array) {
      return array[array.length - 1];
  }

   private final Map<String, Values> valuesByClassname = new LinkedHashMap<>();

   class Values {

      int points = 0,
            reachablePoints = 0,
            currentPoints = 0;

      public Values() {}
      public Values(int currentPoints) {
         this.currentPoints = currentPoints;
      }
      public int getPoints() {
         return points;
      }

      public int getReachablePoints() {
         return reachablePoints;
      }
      public Values set (int currentPoints){
         this.currentPoints = currentPoints;
         return this;
      }
   }

   public void clear() {
      valuesByClassname.clear();
   }
   @Override
   public void testFinished(final Description description) throws Exception {
      final Values values = addDescription(description);
      values.points += values.currentPoints;
      values.reachablePoints += values.currentPoints;
   }

   @Override
   public void testFailure(final Failure failure) throws Exception {
      final Values values = addDescription(failure.getDescription());
      // Ugly: testFinished will be called in any case. So we defeat
      // adding reached points in this method.
      values.points -= values.currentPoints;
   }

   Values addDescription(final Description description) {

      final String testClassName =  last(description.getClassName().split("\\."));
      
      final Marking marking = description.getAnnotation(Marking.class);
      final int reacheablePoints = null == marking? 1: marking.points();
      
      return valuesByClassname.compute(testClassName, (c, v) -> {return v == null? new Values(reacheablePoints): v.set(reacheablePoints);});
   }

   public String getPointSumms() {
      return getPointSumms("Sum");
   }
   public String getPointSumms(final String exerciseTitle) {
      final StringBuffer ret = new StringBuffer();

      int reachablePointSum = 0,
            pointSum = 0,
            maxTestTitleLength = exerciseTitle.length();

      for (Map.Entry<String, Values> entry: valuesByClassname.entrySet()) {
         final Values value = entry.getValue();
         pointSum += value.points;
         reachablePointSum += value.reachablePoints;
         maxTestTitleLength = Math.max(maxTestTitleLength, entry.getKey().length());
      }
      final String formatString = "%" + maxTestTitleLength + "s";
      ret.append(String.format(formatString, exerciseTitle) + ": " + pointSum + "/" + reachablePointSum + "\n");
      for (Map.Entry<String, Values> entry: valuesByClassname.entrySet()) {
         ret.append(String.format(formatString, entry.getKey()));
         ret.append(": ");
         final Values value = entry.getValue();
         pointSum += value.points;
         ret.append(value.points);
         ret.append('/');
         ret.append(value.reachablePoints);
         reachablePointSum += value.reachablePoints;
         ret.append("\n");
      }
      return ret.toString();
   }
   //      @Override
   //      public void testRunStarted(Description description) throws Exception {
   //         super.testRunStarted(description);
   //         System.out.println("testRunStarted" + description);
   //      }
   //
   //   @Override
   //   public void testRunFinished(Result result) throws Exception {
   //      super.testRunFinished(result);
   //      System.out.println("testRunFinished");
   //   }
   //
   //   @Override
   //   public void testStarted(Description description) throws Exception {
   //      super.testStarted(description);
   //      System.out.println("testStarted");
   //   }
   //
   //   @Override
   //   public void testAssumptionFailure(Failure failure) {
   //      super.testAssumptionFailure(failure);
   //      System.out.println("testAssumptionFailure");
   //   }
   //
   //   @Override
   //   public void testIgnored(Description description) throws Exception {
   //      super.testIgnored(description);
   //      System.out.println("testIgnored");
   //   }   
}
