package de.hdm_stuttgart.mi.unitmarking.xsd;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.jdom2.Document;
import org.jdom2.JDOMException;
import org.jdom2.ProcessingInstruction;
import org.jdom2.filter.Filters;
import org.jdom2.input.SAXBuilder;
import org.jdom2.input.sax.XMLReaders;
import org.jdom2.xpath.XPathExpression;
import org.jdom2.xpath.XPathFactory;

import java.io.File;
import java.io.IOException;
import java.util.Arrays;
import java.util.Comparator;
import java.util.List;

/**
 * Evaluate a given set of XML document instances
 *
 */
class CreateTestClass {

	private static Logger log = LogManager.getLogger(CreateTestClass.class);
	private final String xmlTestFileDir;

	/**
	 *
	 * @param xmlTestFileDir Directory containing XML instances to be processed.
	 */
	CreateTestClass(final String xmlTestFileDir) {

		this.xmlTestFileDir = xmlTestFileDir;

		System.out.println("private final XmlTestSupport xmlTestSupport = new XmlTestSupport(\""+
				xmlTestFileDir +"\");\n");

		Arrays.stream(new File(xmlTestFileDir).listFiles(path -> path.getPath().endsWith(".xml"))).
				sorted(Comparator.comparing(File::toString)).
				forEach(this::readTestHeader);
	}

	private void readTestHeader(final File instanceFilename) {

		final List<ProcessingInstruction> xmlTestList = InstanceSetEvaluation.getProcessingInstructions(instanceFilename);

		switch (xmlTestList.size()) {
			case 0:
				log.info("No 'xmlTest PI found, possible dependency file");
				break;
			case 1:
				writeTest(instanceFilename.getName(), xmlTestList.get(0));
				break;
			default:
				log.info("Found " + xmlTestList.size() + "' annotations in file '" +  instanceFilename.getPath() + "'\n");
		}
	}

	private void writeTest(final String fileName, final ProcessingInstruction pi) {

		final int points = Integer.parseInt(pi.getPseudoAttributeValue("points"));
		final boolean expectedToBeValid = Boolean.parseBoolean(pi.getPseudoAttributeValue("expectedToBeValid"));

		System.out.println("@Test @Marking(points = " + points + ")");

		final String fileBaseName = fileName.replaceFirst("[.][^.]+$", "");
		System.out.println("public void test_" + fileBaseName + "() {");


		if (expectedToBeValid) {
			System.out.println("    xmlTestSupport.assertValid(\"" + fileName + "\");");
		} else {
			System.out.println("    xmlTestSupport.assertInvalid(\"" + fileName + "\");");
		}

		final String  preconditionValidFilename = pi.getPseudoAttributeValue("preconditionValid");

		if (null != preconditionValidFilename) {
			System.out.println("    xmlTestSupport.assertValid(\"" + preconditionValidFilename + "\");");
		}
		System.out.println("}\n");
	}
}
