<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:d="http://docbook.org/ns/docbook"
  xmlns:m="http://www.w3.org/1998/Math/MathML"
  xmlns="http://www.w3.org/1999/xhtml"
  exclude-result-prefixes="d m"
  version="1.1">
  
  <xsl:import href="../../docbook-xsl/xhtml5/docbook.xsl"/>
  <xsl:import href="../../docbook-xsl/epub3/epub3-element-mods.xsl"/>
  <xsl:include href="../Common/common.xsl"/>  
  
  <xsl:template name="user.head.content">
    <xsl:param name="node"/>
    <xsl:copy-of select="$toggleDisplayScript"/>    
  </xsl:template>
    
  <xsl:template match="d:videoobject" priority="1">
    <video controls="controls" preload="auto">
      <xsl:attribute name="title">
        <xsl:value-of select="normalize-space(../../d:title)"/>
      </xsl:attribute>
      
      <xsl:variable name="hdm_ImageFilename"> <!-- hdm_ prefix: do not mess with docbook! -->
        <xsl:call-template name="mediaobject.filename">
          <xsl:with-param name="object" select="."/>
        </xsl:call-template>
      </xsl:variable>
      <source src="{$hdm_ImageFilename}.mp4" type='video/mp4'/>
      <xsl:text>Your Browser does not support the video tag</xsl:text>
    </video>
  </xsl:template>
  
  <!-- Correct package.opf manifest file -->
  <xsl:template match="*" mode="package.manifest" priority="1">
    
    <xsl:variable name="is.chunk">
      <xsl:call-template name="chunk">
        <xsl:with-param name="node" select="."/>
      </xsl:call-template>
    </xsl:variable>
    
    <xsl:if test="$is.chunk != 0">
      <xsl:variable name="href">
        <xsl:call-template name="href.target.with.base.dir">
          <xsl:with-param name="context" select="/"/>
        </xsl:call-template>
      </xsl:variable>
      
      <xsl:variable name="id" select="concat($epub.package.id.prefix, generate-id())"/>
      
      <xsl:variable name="properties.set">
        <xsl:call-template name="svg.property"/>
        <xsl:text> </xsl:text>
        <xsl:call-template name="mathml.property"/>
        <xsl:text> scripted</xsl:text>
      </xsl:variable>
      
      <xsl:variable name="properties" select="normalize-space($properties.set)"/>
      
      <xsl:element namespace="{$opf.namespace}" name="item">
        <xsl:attribute name="id">
          <xsl:value-of select="$id"/>
        </xsl:attribute>
        <xsl:attribute name="href">
          <xsl:value-of select="$href"/>
        </xsl:attribute>
        <xsl:attribute name="media-type">application/xhtml+xml</xsl:attribute>
        <xsl:if test="string-length($properties) != 0">
          <xsl:attribute name="properties">
            <xsl:value-of select="$properties"/>
          </xsl:attribute>
        </xsl:if>
      </xsl:element>
    </xsl:if>  
    <xsl:apply-templates mode="package.manifest"/>
    
  </xsl:template>
  
  <!-- Provide supplementary items for package.opf file -->
  

  <xsl:template name="targetComment">
    <xsl:param name="comment"/>
    <xsl:value-of select="$n"/>
    <xsl:value-of select="$n"/>
    <xsl:comment>
      <xsl:value-of select="$comment"/>
    </xsl:comment>
  </xsl:template>
  
  
  <xsl:template name="manifest.toc" priority="1">
    <xsl:variable name="toc.params">
      <xsl:call-template name="find.path.params">
        <xsl:with-param name="node" select="."/>
        <xsl:with-param name="table" select="normalize-space($generate.toc)"/>
      </xsl:call-template>
    </xsl:variable>
    
    <xsl:if test="contains($toc.params, 'toc')">
      <xsl:element namespace="{$opf.namespace}" name="item">
        <xsl:attribute name="id">
          <xsl:value-of select="$epub.html.toc.id"/>
        </xsl:attribute>
        <xsl:attribute name="properties">nav scripted</xsl:attribute>
        <xsl:attribute name="media-type">
          <xsl:value-of select="$epub.xhtml.mediatype"/>
        </xsl:attribute>
        <xsl:attribute name="href">
          <xsl:call-template name="toc-href">
            <xsl:with-param name="node" select="."/>
          </xsl:call-template>
        </xsl:attribute>        
      </xsl:element>
    </xsl:if>  
  </xsl:template>
  
  
  <xsl:template name="user.manifest.items" priority="1">
    
    <xsl:call-template name="targetComment">
      <xsl:with-param name="comment" select="'Derived video data'"/>
    </xsl:call-template>    
    <xsl:apply-templates select="//d:videodata" mode="hdmuserOpf"/>
    
    <xsl:call-template name="targetComment">
      <xsl:with-param name="comment" select="'Docbook distribution icons'"/>
    </xsl:call-template>    
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-002" href="images/prev.gif" media-type="image/gif"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-003" href="images/tip.gif" media-type="image/gif"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-004" href="images/callouts/5.gif" media-type="image/gif"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-005" href="images/callouts/2.gif" media-type="image/gif"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-006" href="images/callouts/8.gif" media-type="image/gif"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-007" href="images/callouts/14.gif" media-type="image/gif"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-008" href="images/callouts/10.gif" media-type="image/gif"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-009" href="images/callouts/13.gif" media-type="image/gif"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-010" href="images/callouts/12.gif" media-type="image/gif"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-011" href="images/callouts/11.gif" media-type="image/gif"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-012" href="images/callouts/9.gif" media-type="image/gif"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-013" href="images/callouts/15.gif" media-type="image/gif"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-014" href="images/callouts/7.gif" media-type="image/gif"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-015" href="images/callouts/3.gif" media-type="image/gif"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-016" href="images/callouts/1.gif" media-type="image/gif"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-017" href="images/callouts/4.gif" media-type="image/gif"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-018" href="images/callouts/6.gif" media-type="image/gif"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-019" href="images/caution.gif" media-type="image/gif"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-020" href="images/up.gif" media-type="image/gif"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-021" href="images/warning.gif" media-type="image/gif"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-022" href="images/note.gif" media-type="image/gif"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-023" href="images/important.gif" media-type="image/gif"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-024" href="images/next.gif" media-type="image/gif"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-025" href="images/home.gif" media-type="image/gif"/>
    
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-100" href="images/tip.svg" media-type="image/svg"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-101" href="images/prev.svg" media-type="image/svg"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-102" href="images/callouts/21.svg" media-type="image/svg"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-103" href="images/callouts/3.svg" media-type="image/svg"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-104" href="images/callouts/15.svg" media-type="image/svg"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-105" href="images/callouts/19.svg" media-type="image/svg"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-106" href="images/callouts/18.svg" media-type="image/svg"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-107" href="images/callouts/9.svg" media-type="image/svg"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-108" href="images/callouts/4.svg" media-type="image/svg"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-109" href="images/callouts/24.svg" media-type="image/svg"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-110" href="images/callouts/6.svg" media-type="image/svg"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-111" href="images/callouts/29.svg" media-type="image/svg"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-112" href="images/callouts/27.svg" media-type="image/svg"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-113" href="images/callouts/16.svg" media-type="image/svg"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-114" href="images/callouts/12.svg" media-type="image/svg"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-115" href="images/callouts/17.svg" media-type="image/svg"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-116" href="images/callouts/30.svg" media-type="image/svg"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-117" href="images/callouts/22.svg" media-type="image/svg"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-118" href="images/callouts/20.svg" media-type="image/svg"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-119" href="images/callouts/28.svg" media-type="image/svg"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-120" href="images/callouts/26.svg" media-type="image/svg"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-121" href="images/callouts/10.svg" media-type="image/svg"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-122" href="images/callouts/11.svg" media-type="image/svg"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-123" href="images/callouts/14.svg" media-type="image/svg"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-124" href="images/callouts/7.svg" media-type="image/svg"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-125" href="images/callouts/8.svg" media-type="image/svg"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-126" href="images/callouts/5.svg" media-type="image/svg"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-127" href="images/callouts/23.svg" media-type="image/svg"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-128" href="images/callouts/13.svg" media-type="image/svg"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-129" href="images/callouts/25.svg" media-type="image/svg"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-130" href="images/callouts/2.svg" media-type="image/svg"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-131" href="images/callouts/1.svg" media-type="image/svg"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-132" href="images/draft.svg" media-type="image/svg"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-133" href="images/note.svg" media-type="image/svg"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-134" href="images/colorsvg/tip.svg" media-type="image/svg"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-135" href="images/colorsvg/prev.svg" media-type="image/svg"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-136" href="images/colorsvg/note.svg" media-type="image/svg"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-137" href="images/colorsvg/important.svg" media-type="image/svg"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-138" href="images/colorsvg/up.svg" media-type="image/svg"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-139" href="images/colorsvg/caution.svg" media-type="image/svg"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-140" href="images/colorsvg/next.svg" media-type="image/svg"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-141" href="images/colorsvg/home.svg" media-type="image/svg"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-142" href="images/colorsvg/warning.svg" media-type="image/svg"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-143" href="images/important.svg" media-type="image/svg"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-144" href="images/up.svg" media-type="image/svg"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-145" href="images/caution.svg" media-type="image/svg"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-146" href="images/next.svg" media-type="image/svg"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-147" href="images/home.svg" media-type="image/svg"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-148" href="images/warning.svg" media-type="image/svg"/>
    
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-200" href="images/prev.png" media-type="image/png"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-201" href="images/draft.png" media-type="image/png"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-202" href="images/warning.png" media-type="image/png"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-203" href="images/up.png" media-type="image/png"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-204" href="images/tip.png" media-type="image/png"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-205" href="images/note.png" media-type="image/png"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-206" href="images/callouts/4.png" media-type="image/png"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-207" href="images/callouts/15.png" media-type="image/png"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-208" href="images/callouts/3.png" media-type="image/png"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-209" href="images/callouts/2.png" media-type="image/png"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-210" href="images/callouts/7.png" media-type="image/png"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-211" href="images/callouts/5.png" media-type="image/png"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-212" href="images/callouts/14.png" media-type="image/png"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-213" href="images/callouts/8.png" media-type="image/png"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-214" href="images/callouts/11.png" media-type="image/png"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-215" href="images/callouts/13.png" media-type="image/png"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-216" href="images/callouts/9.png" media-type="image/png"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-217" href="images/callouts/12.png" media-type="image/png"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-218" href="images/callouts/6.png" media-type="image/png"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-219" href="images/callouts/10.png" media-type="image/png"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-220" href="images/callouts/1.png" media-type="image/png"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-221" href="images/toc-blank.png" media-type="image/png"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-222" href="images/next.png" media-type="image/png"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-223" href="images/toc-plus.png" media-type="image/png"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-224" href="images/important.png" media-type="image/png"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-225" href="images/blank.png" media-type="image/png"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-226" href="images/home.png" media-type="image/png"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-227" href="images/annot-close.png" media-type="image/png"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-228" href="images/caution.png" media-type="image/png"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-229" href="images/annot-open.png" media-type="image/png"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-230" href="images/toc-minus.png" media-type="image/png"/>
    
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-300" href="images/caution.tif" media-type="image/tif"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-301" href="images/important.tif" media-type="image/tif"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-302" href="images/warning.tif" media-type="image/tif"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-303" href="images/note.tif" media-type="image/tif"/>
    <item xmlns="http://www.idpf.org/2007/opf"  id="dbook-icons-304" href="images/tip.tif" media-type="image/tif"/>
        
  </xsl:template>
  
  <xsl:template match="d:videodata" mode="hdmuserOpf">
    <xsl:variable name="baseFileName">
      <xsl:call-template name="relative-uri">
        <xsl:with-param name="filename" select="@fileref"/>
      </xsl:call-template>
    </xsl:variable>

    <xsl:element namespace="{$opf.namespace}" name="item">
      <xsl:attribute name="id">
        <xsl:value-of select="generate-id()"/>
        <xsl:text>-HDMEXTRA-mp4</xsl:text>
      </xsl:attribute>
      <xsl:attribute name="href">
        <xsl:value-of select="$baseFileName"/>
        <xsl:text>.mp4</xsl:text>
      </xsl:attribute>
      <xsl:attribute name="media-type">video/mp4</xsl:attribute>
    </xsl:element>

    <xsl:element namespace="{$opf.namespace}" name="item">
      <xsl:attribute name="id">
        <xsl:value-of select="generate-id()"/>
        <xsl:text>-HDMEXTRA-png</xsl:text>
      </xsl:attribute>
      <xsl:attribute name="href">
        <xsl:value-of select="$baseFileName"/>
        <xsl:text>.png</xsl:text>
      </xsl:attribute>
      <xsl:attribute name="media-type">video/mp4</xsl:attribute>
    </xsl:element>
  </xsl:template>
  
  <xsl:template match="*" mode="hdmuserOpf">
    <xsl:text>ERROR: No template available for element '</xsl:text>
    <xsl:value-of select="name(.)"/>
    <xsl:text>' in opf mode.</xsl:text>
  </xsl:template>
  
  <!--  <xsl:template match="d:imagedata" mode="hdmuserOpf">
    </xsl:template>
    
  -->  
</xsl:stylesheet>
