package de.hdm_stuttgart.mi.bb;

import de.hdm_stuttgart.mi.bb.model.Topic;
import org.glassfish.jersey.client.ClientConfig;
import org.glassfish.jersey.jackson.JacksonFeature;
import org.jdom2.Document;
import org.jdom2.Element;
import org.jdom2.JDOMException;
import org.jdom2.input.SAXBuilder;
import org.jdom2.output.Format;
import org.jdom2.output.XMLOutputter;

import javax.ws.rs.client.*;
import javax.ws.rs.core.HttpHeaders;
import javax.ws.rs.core.MediaType;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.Locale;
import java.util.Map;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import java.util.logging.Logger;
import java.util.logging.Level;

public class Id2Topic {

    static {
        final Logger mongoLogger = Logger.getLogger( "org.mongodb.driver" );
        mongoLogger.setLevel(Level.SEVERE); // e.g. or Log.WARNING, etc.

        final String
            configPathName= System.getProperty("user.home"),
            configFileName = "bbtopics",
            fqPathname = configPathName + '/' + configFileName + ".properties";

        final File file = new File(configPathName);

        ResourceBundle rb = null;
        try {
            final URL[] urls = new URL[]{file.toURI().toURL()};
            final ClassLoader loader = new URLClassLoader(urls);
            rb = ResourceBundle.getBundle(configFileName, Locale.getDefault(), loader);
        } catch (final MalformedURLException | MissingResourceException e) {
            System.err.println("Unable to open URI " + fqPathname);
            System.exit(1);
        }
        config = rb;
    }

    static private final ResourceBundle config;

    static private final
    ModifyNodebbTopic db = new ModifyNodebbTopic(
            config.getString("mongoUri"),
            config.getString("nodebbDb"));

    static private final Client client = ClientBuilder.newClient(new ClientConfig().register(new JacksonFeature()));

    static private final String bearerToken = config.getString("nodebbBearerToken");

    static private final String
            KEY_PAYLOAD = "payload",
            KEY_TOPICDATA = "topicData";

    public static void main(String[] args) {


        final SAXBuilder saxBuilder = new SAXBuilder();

        if (0 == args.length) {
            System.err.println("Usage: id2topic <docbook2topic basefilename>");
        } else {
            final File inputFile = new File(args[0] + ".xml");

            try {
                final Document document = saxBuilder.build(inputFile);
                handleDocbok2idMap(document.getRootElement());

                final Format outFormat = Format.getPrettyFormat();

                // Serialize to console
                final XMLOutputter printer = new XMLOutputter(outFormat);

                final String outputFileName = args[0] + ".tid.xml";
                final FileWriter outputFile = new FileWriter(outputFileName);
                printer.output(document.getRootElement(), outputFile);
            } catch (JDOMException e) {
                System.err.println("Unable to parse " + inputFile);
            } catch (IOException e) {
                System.err.println("Unable to read " + inputFile);
            }
        }
    }

    static private void handleDocbok2idMap(final Element map) {
        String mapPrefix = map.getAttributeValue("prefix");

        for (final Element entry : map.getChildren("entry")) {
            final int tid = createTopic(
                    mapPrefix + ":" + entry.getAttributeValue("id"),
                    Integer.valueOf(entry.getAttributeValue("forumId")),
                    entry.getChildText("title"),
                    entry.getChildText("content"));
            entry.setAttribute("tid", "" + tid);
        }
    }

    static private int createTopic(String hdmId, int categoryId, String title, String content) {

        int tid = db.hdmIdExists(hdmId);
        if (tid < 0) { // No related topic
            final WebTarget topicsTarget = client.target(config.getString("nodebbWriteApiTopicsEndpoint"));

            final Topic topic = new Topic(categoryId, title, content);

            final Entity jsonTopic = Entity.json(topic);
            System.out.println("Feeding in Topic " + topic);
            final Map<String, Object> response = (Map<String, Object>)
                topicsTarget.request(MediaType.APPLICATION_JSON).
                header(HttpHeaders.AUTHORIZATION, "Bearer " + bearerToken).
                accept(MediaType.TEXT_PLAIN_TYPE).
                post(jsonTopic, Map.class);

            final Map<String, Object> topicData = (Map<String, Object>)
                ((Map<String, Object>)(response.get(KEY_PAYLOAD))).
                get(KEY_TOPICDATA);

            int newTid = (int) topicData.get("tid");
            db.addHdmId(newTid, hdmId);
            return newTid;
        } else {
            System.out.println("hdmId '" + hdmId + "' already present, just adding tid");
            return tid;
            // Possibly aligning title and content with lecture notes?
            // Difficult: Incomplete NodeBB write API --> required MongoDB fiddling

            /*
            final WebTarget hdmTtopicTarget =
                    client.target(config.getString("nodebbReadApiTopicsEndpoint") + "/" + tid);

            final Response response = hdmTtopicTarget.request(MediaType.APPLICATION_JSON_TYPE).get();

            final Map<String, Object> hdmTopicMap = response.readEntity(Map.class);
            System.out.println("-----------------------Topic [tid=" + tid + "] data:");
            for (String k : hdmTopicMap) {
                System.out.println("key '" + k + "', val '" + m.get(k) + "'");
            }
             */
        }
    }
}
