<?xml version="1.0" encoding="iso-8859-1"?>

<xsl:stylesheet 
  xmlns:fo="http://www.w3.org/1999/XSL/Format" 
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="2.0"
  xmlns:kk="http://www.hdm-stuttgart.de/xmleditor/schema/kkabrechnung">
  
  <xsl:include href="common.xsl"/>

  <xsl:output indent="yes" method="xml"/>
  
  
  <xsl:template match="/">
    <fo:root
      language="de"
      hyphenate="true"
      hyphenation-character="-"
      hyphenation-push-character-count="3"
      hyphenation-remain-character-count="3"
      font-family="Times">
      <!--____________________ Layout Master Set _____________________-->
      
      <fo:layout-master-set>
        <fo:simple-page-master master-name="pageLayout" 
          page-height="297mm" page-width="210mm"
          margin-top="20mm" margin-bottom="20mm"		
          margin-left="20mm" margin-right="20mm">
          <fo:region-body margin-top="20mm"/>
          <fo:region-before extent="20mm"/>
        </fo:simple-page-master>
      </fo:layout-master-set>
      
      <!--____________________ Page sequence content _________________-->
      
      <fo:page-sequence master-reference="pageLayout">
        <fo:static-content flow-name="xsl-region-before">
          <fo:block font-size="36pt" font-weight="bold">
          </fo:block>
        </fo:static-content>
        <fo:flow flow-name="xsl-region-body" font-size="12pt">
            <xsl:apply-templates select="kk:abrechnung"/>
        </fo:flow>
      </fo:page-sequence>
    </fo:root>
  </xsl:template>

  <xsl:template match="kk:abrechnung">
      
      <xsl:apply-templates select="kk:unfaelle"/>
      <fo:block font-size="16pt" font-weight="bold">Rechnungspositionen:</fo:block>
      
    <fo:table font-size="10pt" border-style="solid" border-width="0.5pt">
      <fo:table-column column-width="34mm"/>
      <xsl:for-each select="kk:berechtigte/*">
        <fo:table-column column-width="35mm"/>
      </xsl:for-each>
      <fo:table-body>
        <fo:table-row>
          <fo:table-cell border-right=".5pt solid black">
            <fo:block>Name</fo:block>
          </fo:table-cell>
          <xsl:apply-templates select="kk:berechtigte/*" mode="kennzeichnung"/>
        </fo:table-row>

        <fo:table-row>
          <fo:table-cell border-top=".5pt solid black" border-right=".5pt solid black">
            <fo:block>Kostenart</fo:block>
          </fo:table-cell>
          <xsl:for-each select="kk:berechtigte/*">
            <fo:table-cell border-top=".5pt solid black">
              <xsl:if test="position() &lt; last()">
                <xsl:attribute name="border-right">.5pt solid black</xsl:attribute>
              </xsl:if>
              <fo:table>
                <fo:table-column column-width="17mm"/>
                <fo:table-column column-width="5mm"/>
                <fo:table-column column-width="12mm"/>
                <fo:table-body>
                  <fo:table-row>
                    <fo:table-cell>
                      <fo:block>Datum</fo:block>
                    </fo:table-cell>
                    <fo:table-cell>
                      <fo:block>Nr</fo:block>
                    </fo:table-cell>
                    <fo:table-cell>
                      <fo:block text-align="end">Euro</fo:block>
                    </fo:table-cell>
                  </fo:table-row>
                </fo:table-body>
              </fo:table>
            </fo:table-cell>
          </xsl:for-each>
        </fo:table-row>
        
        <xsl:apply-templates select="kk:berechtigte">
          <xsl:with-param name="typKey">rezept</xsl:with-param>
        </xsl:apply-templates>
        
        <xsl:apply-templates select="kk:berechtigte">
          <xsl:with-param name="typKey">ambulant</xsl:with-param>
        </xsl:apply-templates>
        
        <xsl:apply-templates select="kk:berechtigte">
          <xsl:with-param name="typKey">zahn</xsl:with-param>
        </xsl:apply-templates>
        
        <xsl:apply-templates select="kk:berechtigte">
          <xsl:with-param name="typKey">heilmittel</xsl:with-param>
        </xsl:apply-templates>
        
        <xsl:apply-templates select="kk:berechtigte">
          <xsl:with-param name="typKey">stationaer</xsl:with-param>
        </xsl:apply-templates>
        
        <!--        <xsl:if test=".//ambulant">
          <fo:table-row>
            <fo:table-cell border-top=".5pt solid black" border-right=".5pt solid black">
              <fo:block>ambulante Behandlung</fo:block>
            </fo:table-cell>
            <xsl:apply-templates
              select="kk:antragsteller/person|ehepartner/person|kind/person" mode="ambulant"/>
          </fo:table-row>
        </xsl:if>

        <xsl:if test=".//rezept">
          <fo:table-row>
            <fo:table-cell border-top=".5pt solid black" border-right=".5pt solid black">
              <fo:block>Rezepte</fo:block>
            </fo:table-cell>
            <xsl:apply-templates
              select="kk:antragsteller/person|ehepartner/person|kind/person" mode="rezept"/>
          </fo:table-row>
        </xsl:if>

        <xsl:if test=".//heilmittel">
          <fo:table-row>
            <fo:table-cell border-top=".5pt solid black" border-right=".5pt solid black">
              <fo:block>Heilmittel</fo:block>
            </fo:table-cell>
            <xsl:apply-templates
              select="kk:antragsteller/person|ehepartner/person|kind/person" mode="heilmittel"/>
          </fo:table-row>
        </xsl:if>

        <xsl:if test=".//stationaer">
          <fo:table-row>
            <fo:table-cell border-top=".5pt solid black" border-right=".5pt solid black">
              <fo:block>Stationre Behandlung</fo:block>
            </fo:table-cell>
            <xsl:apply-templates
              select="kk:antragsteller/person|ehepartner/person|kind/person" mode="stationaer"/>
          </fo:table-row>
        </xsl:if>

        <xsl:if test=".//zahn">
          <fo:table-row>
            <fo:table-cell border-top=".5pt solid black" border-right=".5pt solid black">
              <fo:block>Zahnbehandlung</fo:block>
            </fo:table-cell>
            <xsl:apply-templates
              select="kk:antragsteller/person|ehepartner/person|kind/person" mode="zahn"/>
          </fo:table-row>
        </xsl:if>
        
        -->
        <fo:table-row>
          <fo:table-cell border-top=".5pt solid black" border-right=".5pt solid black">
            <fo:block>Gesamt</fo:block>
          </fo:table-cell>
          <xsl:apply-templates select="kk:berechtigte/*" mode="gesamt"/>
        </fo:table-row>

        <fo:table-row>
          <fo:table-cell
            number-columns-spanned="{count(kk:berechtigte/*)}"
            border-top=".5pt solid black" border-right=".5pt solid black">
            <fo:block text-align="end">Gesamtsumme:</fo:block>
          </fo:table-cell>
          
          <fo:table-cell
            border-top=".5pt solid black">
            <fo:block font-weight="bold" text-align="end">
              <xsl:value-of select="format-number(sum(kk:verordnungen/kk:verordnung/@betrag),'.00')"/>
            </fo:block>
          </fo:table-cell>
        </fo:table-row>

      </fo:table-body>
    </fo:table>
    <fo:block space-before="10mm">Hinweis: Das Feld <fo:inline font-weight="bold">"Nr"</fo:inline> bezieht sich auf die fortlaufende Nummer des jeweiligen Belegs in den Anlagen</fo:block>
  </xsl:template>
    

  <xsl:template match="kk:berechtigte">
    <xsl:param name="typKey"/>
    <xsl:if test="//kk:verordnung[@typ=$typKey]">
      <fo:table-row>
      <fo:table-cell border-top=".5pt solid black" border-right=".5pt solid black">
        <fo:block>
          <xsl:call-template name="behandlungsart">
            <xsl:with-param name="typ" select="$typKey"></xsl:with-param>
          </xsl:call-template>
        </fo:block>
      </fo:table-cell>
      <xsl:apply-templates select="*">
        <xsl:with-param name="typKey" select="$typKey"/>
      </xsl:apply-templates>
    </fo:table-row>
    </xsl:if>
  </xsl:template>

  <xsl:template match="kk:berechtigte/*">
    <xsl:param name="typKey"/>
      <fo:table-cell border-top=".5pt solid black" border-right=".5pt solid black">
        <xsl:variable name="verord" select="key('verordnungByBerechtigter', @id)[@typ=$typKey]"/>
        <xsl:choose>
          <xsl:when test="$verord">
            <fo:table>
              <fo:table-column column-width="17mm"/>
              <fo:table-column column-width="5mm"/>
              <fo:table-column column-width="12mm"/>
              <fo:table-body>
                <xsl:for-each select="$verord">
                  <fo:table-row>
                    <fo:table-cell>
                      <fo:block font-weight="bold">
                        <xsl:value-of select="@datum"/>
                      </fo:block>
                        <xsl:if test="@unfall">
                            <fo:block font-weight="bold">
                                <xsl:text>Unfall </xsl:text>
                                <xsl:apply-templates select="key('unfallById', @unfall)" mode="nr" />
                            </fo:block>   
                        </xsl:if>
                    </fo:table-cell>
                    <fo:table-cell>
                      <fo:block font-weight="bold">
                        <xsl:value-of select="@nr"/>
                      </fo:block>
                        
                    </fo:table-cell>
                    <fo:table-cell>
                      <fo:block  font-weight="bold" text-align="end">
                        <xsl:value-of select="@betrag"/>
                      </fo:block>
                    </fo:table-cell>
                  </fo:table-row>
                </xsl:for-each>
              </fo:table-body>
            </fo:table>
          </xsl:when>
          <xsl:otherwise>
            <fo:block/>
          </xsl:otherwise>
        </xsl:choose>
      </fo:table-cell>
  </xsl:template>

  <xsl:template match="kk:antragsteller|kk:ehepartner" mode="kennzeichnung">
    <fo:table-cell border-right=".5pt solid black">
      <fo:block>
        <xsl:value-of select="kk:name"/>
      </fo:block>
    </fo:table-cell>
  </xsl:template>

  <xsl:template match="kk:kind" mode="kennzeichnung">
    <fo:table-cell border-right=".5pt solid black">
      <fo:block>
        <xsl:text>Kind </xsl:text>
        <xsl:value-of select="kk:name"/>
      </fo:block>
    </fo:table-cell>
  </xsl:template>

  <xsl:template match="kk:antragsteller|kk:ehepartner|kk:kind" mode="gesamt">
    <fo:table-cell border-right=".5pt solid black" border-top=".5pt solid black">
      <fo:block font-weight="bold" text-align="end">
        <xsl:value-of select="format-number(sum(key('verordnungByBerechtigter', @id)/@betrag),'0.00')"/>
      </fo:block>
    </fo:table-cell>
  </xsl:template>
    
    
</xsl:stylesheet>

