package hibintro.v7;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.Table;
import javax.persistence.UniqueConstraint;

/**
 *  {@link User} instances providing a non-unique index
 *  to the {@link #getCname()} property for faster database retrieval.
 */
@Entity
@Table(uniqueConstraints={@UniqueConstraint(name="uidKey", columnNames={"uid"})},
      indexes = {@Index(name = "byCname", columnList="cname", unique = false)}
)
public class User {

  int uidNumber;
  @Id
  public int getUidNumber() { return uidNumber; }
  public void setUidNumber(int uidNumber) { this.uidNumber = uidNumber; }

  String uid;
  /**
   * @return The user's unique login name e.g. "goik"
   */
  @Column(nullable=false)
  public String getUid() { return uid; }

  /**
   * @param uid See {@link #getUid()}.
   */
  public void setUid(String uid) { this.uid = uid; }

  String cname;

  /**
   * @return The user's common name e.g. "Martin Goik"
   */
  @Column(nullable = false)
  public String getCname() { return cname; }

  /**
   * @param cname See {@link #getCname()}.
   */
  public void setCname(String cname) { this.cname = cname; }

  protected User() {}
  
  /**
   * @param uidNumber See {@link #getUidNumber()}.
   * @param uid See {@link #getUid()}.
   * @param cname See {@link #getCname()}.
   */
  public User(int uidNumber, String uid, String cname) {
    setUidNumber(uidNumber);
    setUid(uid);
    setCname(cname);
  }
}