package sax;

import java.io.IOException;

import javax.xml.parsers.*;

import org.xml.sax.*;

import sax.stat.v2.MyErrorHandler;

/** Show namespace handling */
public class Namespace {
  
  /**
   * @param argv unused
   * @throws Exception Parser initialization exceptions not handled. */
  public static void main(String argv[]) throws Exception{
    final Namespace val = new Namespace();
    val.parse("Input/Sax/trans.xsl");
  }
  /** Set up parser with namespace awareness
   * @throws SAXException Parsing may fail.
   * @throws ParserConfigurationException Unable to instantiate parser.
   */
  public Namespace()
   throws SAXException, ParserConfigurationException{
      final SAXParserFactory saxPf = SAXParserFactory.newInstance();
      final SAXParser saxParser = saxPf.newSAXParser();
      xmlReader = saxParser.getXMLReader();
      xmlReader.setFeature("http://xml.org/sax/features/namespaces",
          true);
      xmlReader.setFeature(
          "http://xml.org/sax/features/namespace-prefixes", true);
      xmlReader.setContentHandler(namespaceEh);
      xmlReader.setErrorHandler(errorHandler);
  }
  /** Parse a given document
   * @param uri The resource to be parsed
   * @throws IOException Access related problems
   * @throws SAXException Parsing may fail
   */
  public void parse(final String uri) 
    throws IOException, SAXException{
    xmlReader.parse(uri);
  }
  private final XMLReader xmlReader;
  private final NamespaceEventHandler namespaceEh = new NamespaceEventHandler();
  private final MyErrorHandler errorHandler = new MyErrorHandler();
}