package sda.jdbc.intro.auth;

import javax.swing.InputVerifier;
import javax.swing.JLabel;

   /**
    * For each field in a GUI we typically need the following items:
    * <ul>
    *   <li>A label e.g. "Enter Password:"</li>
    *   <li>An input component like {@link javax.swing.JTextField}</li>
    *   <li>A validation method or object like {@link InputVerifier}</li>
    *   <li>A container holding error messages to be presented at GUI level
    *   when an input field value is invalid e.g. "Password too short"</li>
    * </ul>
    * This wrapper allows grouping of the above items.
    * 
    * @author goik
    *
    */
public class UserInputUnit {
   
   final JLabel label;
   final InputVerifierNotify verifier;
   final JLabel errorMessage;
   
   public UserInputUnit(final String guiText, final InputVerifierNotify verifier) {
      this.label = new JLabel(guiText);
      this.verifier = verifier;
      errorMessage = new JLabel();
   }
   
   /**
    * Convenience method. The actual text field receiving
    * user input is contained within the verifier.
    * @return the GUI elements text value.
    */
   public String getText() {
      return verifier.field.getText();
   }
}
